﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Threading;
using System.Collections.ObjectModel;
using System.Net;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml.
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Maximale Daten (16 MB).
        /// </summary>
        private const int MAX_BYTES = 16777216;

        /// <summary>
        /// True, wenn die jeweils erste Messung von der Berechnung ausgeschlossen werden soll.
        /// </summary>
        private bool _bExcludeFirst = false;

        /// <summary>
        /// Definiert die maximale Anzahl der Daten die berechnet werden sollen.
        /// </summary>
        private int _iMaxByte;

        /// <summary>
        /// Hält die Instanz für die Hintergrundverarbetiung.
        /// </summary>
        private BackgroundWorker Worker { get; set; }

        /// <summary>
        /// Liefert die Instanz des Containers für Messresultate.
        /// </summary>
        public CwlbMeasurementModel MeasurementModel { get; private set; }

        /// <summary>
        /// Liefert das Viewmodel der Daten
        /// </summary>
        public CwlbMeasurementView MeasurementView { get; private set; }

        /// <summary>
        /// Intialisiert das Hauptfenster.
        /// </summary>
        public MainWindow()
        {
            // Erstelle den Container für die Darstellung der Messresultate
            MeasurementModel = new CwlbMeasurementModel();
            MeasurementView = new CwlbMeasurementView(MeasurementModel);
            this.DataContext = MeasurementView;

            // Intitialisiere das UI
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung für die Schaltfläche starten
        /// </summary>
        private void Start_Clicked(object sender, RoutedEventArgs e)
        {
            try
            {
                // Check Values
                // Bereitstellen von Daten
                if (string.IsNullOrWhiteSpace(_tbMaxBytes.Text))
                {
                    _iMaxByte = 1048576;
                }
                else
                {
                    _iMaxByte = Convert.ToInt32(_tbMaxBytes.Text);
                }
                if (_iMaxByte > MAX_BYTES)
                {
                    MessageBox.Show(string.Format("Die maximale Anzahl zu verarbeitende Bytes ist mit {0} Bytes festgelegt.", MAX_BYTES), this.Title, MessageBoxButton.OK, MessageBoxImage.Information);
                    return;
                }
                if (_iMaxByte < 64)
                {
                    MessageBox.Show("Die minimale Anzahl zu verarbeitende Bytes ist mit 64 Bytes festgelegt.", this.Title, MessageBoxButton.OK, MessageBoxImage.Information);
                    return;
                }

                // UI sperren
                _gridEntry.IsEnabled = false;
                _tbStatus.Text = "Messung läuft";
                _bExcludeFirst = _chkExcludeFirst.IsChecked ?? default(bool);

                // Hintergrundverarbeitung initialisieren
                CreateBackgroundWorker();
                // Worker Starten
                Worker.RunWorkerAsync();
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        /// <summary>
        /// Erstellt den backgroundworker.
        /// </summary>
        private void CreateBackgroundWorker()
        {
            Worker = new BackgroundWorker();
            Worker.DoWork += new DoWorkEventHandler(Worker_DoWork);
            Worker.ProgressChanged += new ProgressChangedEventHandler(Worker_ProgressChanged);
            Worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(Worker_RunWorkerCompleted);
            Worker.WorkerReportsProgress = true;
            Worker.WorkerSupportsCancellation = true;
        }

        /// <summary>
        /// Eigtenliche Arbeitsmethode des Backgroundworkers.
        /// </summary>
        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            // Initialisieren
            // Die Initialisierung wird synchron gemacht wegen den Veränderungen in den gebundenen Collections.
            MeasurementModel.InitializeMeasurement(Worker, _iMaxByte, _bExcludeFirst);
            // Leeren des Containers für Messresultate
            Worker.ReportProgress(-1);
            Thread.Sleep(500);

            // Durchführen der Tests
            MeasurementModel.Test1DomainInprocess(1);
            MeasurementModel.Test2DomainsInprocess(26);
            MeasurementModel.Test1MachineOutOfProcess(51);
            MeasurementModel.Test2MachinesOutOfProcess(76);

            // Die Erignisbehandlung für das Beenden des Workers wird autoamtisch aufgerufen.
        }

        /// <summary>
        /// Ereignisbehandlung für das Melden von Veränderungen von Daten im Messprozess.
        /// </summary>
        private void Worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Leeren der vorangehenden Resultate
            if (e.ProgressPercentage == -1)
            {
                MeasurementView.Clear();
                _pgrWork.Value = 0;
                return;
            }
            else if (e.ProgressPercentage == 1)
            {
            }
            else
            {
                // Fortschritt nachführen
                if (e.UserState is CwlbModelChangedEventArgs)
                {
                    MeasurementView.Progress(e.UserState as CwlbModelChangedEventArgs);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Beenden der hintergrundverarbeitung.
        /// </summary>
        private void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // UI entsperren
            _gridEntry.IsEnabled = true;
            _tbStatus.Text = "Bereit";
            _pgrWork.Value = 0;
        }

        /// <summary>
        /// Ereignisbehandlung zum Prüfen der Verbindung zu einem Server
        /// </summary>
        private void Button_Check_ServerClicked(object sender, RoutedEventArgs e)
        {
            try
            {
                // Prüfen der eingegebenen Daten
                string strRemoteMachineName = MeasurementView.RemoteMachineName;
                IPAddress objAddress = CwlbMeasurementModel.GetIpAddressRemoteMachine(MeasurementModel.IpAddressRemoteMachine, ref strRemoteMachineName);
                if (IPAddress.Equals(objAddress, IPAddress.None))
                {
                    MessageBox.Show("Die gesuchte Maschine wurde nicht gefunden.", Title, MessageBoxButton.OK, MessageBoxImage.Information);
                }
                else
                {
                    MeasurementView.RemoteMachineName = strRemoteMachineName;
                    MeasurementView.IpAddressRemoteMachine = objAddress.ToString();
                }
            }
            catch(Exception excObject)
            {
                MessageBox.Show(excObject.Message, Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
    }
}
