﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Threading;
using System.Net;
using System.Net.NetworkInformation;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace WeroSoft.Samples
{
    public class CwlbMeasurementModel
    {
        /// <summary>
        /// Anzahl Messungen für die Durchschnittsbildung.
        /// </summary>
        private const int MAX_LOOP = 10;

        /// <summary>
        /// Name des Messfalles für die Messung von einer Domäne.
        /// </summary>
        public const string InProcess = "InProcess";

        /// <summary>
        /// Name des Messfalls für die Messung von 2 Domänen in einem gleichen Prozessraum.
        /// </summary>
        public const string DomainInProcess = "DomainInProcess";

        /// <summary>
        /// Name des Messfalls für die Messung von 2 Prozessen in einer Maschine.
        /// </summary>
        public const string OutOfProcessLocal = "OutOfProcessLocal";

        /// <summary>
        /// Name des Messfalls für die Messung von 2 Prozessen auf zwei Maschinen.
        /// </summary>
        public const string OutOfProcessRemote = "OutOfProcessRemote";

        /// <summary>
        /// Definiert den verwendeten Zufallszahlengenerator für die Daten
        /// </summary>
        private Random _objRandom = new Random();

        /// <summary>
        /// Definiert die verwendete Stopuhr für dei Messungen.
        /// </summary>
        private Stopwatch _objStopwatch = Stopwatch.StartNew();

        /// <summary>
        /// Liefert das Objekt für die Hintergrundsverarbeitung.
        /// </summary>
        public BackgroundWorker Worker { get; private set; }

        /// <summary>
        /// Liefert den Namen der lokalen Maschine
        /// </summary>
        public string LocalMachineName { get; private set; }

        /// <summary>
        /// Liefert die IP Adresse der lokalen Maschine
        /// </summary>
        public string IpAddressLocalMachine { get; private set; }

        /// <summary>
        /// Liefert oder definiert den Name der entfernten Maschine.
        /// </summary>
        public string RemoteMachineName { get; set; }

        /// <summary>
        /// Liefert oder definiert die IP-Adresse der entfernten Machine.
        /// </summary>
        public string IpAddressRemoteMachine { get; set; }

        /// <summary>
        /// Liefert die Liste der Datenlängen, die verarbeitet werden
        /// </summary>
        public ObservableCollection<int> DataLengthCollection { get; set; }

        /// <summary>
        /// Liefert das Verzeichnis der Messdatenreihen. Jeder Eintrag dient zur Erfassung der Messungen
        /// einer Technologie.
        /// </summary>
        public Dictionary<string,CwlbMeasurementCase> MeasurementCaseCollection { get; set; }

        /// <summary>
        /// Initialisiert eine neue Instanz des Types CwlbMeasurement.
        /// </summary>
        public CwlbMeasurementModel()
        {
            LocalMachineName = Environment.MachineName;
            IpAddressLocalMachine = string.Empty;
            NetworkInterface[] aNics = NetworkInterface.GetAllNetworkInterfaces();
            foreach (NetworkInterface objNic in aNics)
            {
                IPInterfaceProperties objNicProperties = objNic.GetIPProperties();
                if (objNicProperties.UnicastAddresses.Count > 0)
                {
                    if (objNicProperties.UnicastAddresses[0].Address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                    {
                        if (IpAddressLocalMachine.Length > 0)
                        {
                            IpAddressLocalMachine += " | ";
                        }
                        IpAddressLocalMachine += objNicProperties.UnicastAddresses[0].Address.ToString();
                    }
                }
            }
        }

        /// <summary>
        /// Intialisierung der Messung.
        /// </summary>
        public void InitializeMeasurement(BackgroundWorker objWorker, int iMaxByte, bool bExcludeFirst)
        {
            // Referenz des Workers übernehmen
            Worker = objWorker;
            // Erstellen der Datencontainer
            DataLengthCollection = new ObservableCollection<int>();
            MeasurementCaseCollection = new Dictionary<string, CwlbMeasurementCase>();
            // Konfigurieren der Messfälle
            MeasurementCaseCollection.Add(InProcess, new CwlbMeasurementCase(InProcess));
            MeasurementCaseCollection.Add(DomainInProcess, new CwlbMeasurementCase(DomainInProcess));
            MeasurementCaseCollection.Add(OutOfProcessLocal, new CwlbMeasurementCase(OutOfProcessLocal));
            MeasurementCaseCollection.Add(OutOfProcessRemote, new CwlbMeasurementCase(OutOfProcessRemote));

            // Berechnen der Datenschritte
            for (int iDataLength = 64; iDataLength <= iMaxByte; iDataLength *= 2)
            {
                DataLengthCollection.Add(iDataLength);
                foreach (var item in MeasurementCaseCollection)
                {
                    item.Value.Measurement.Add(new CwlbMeasurementDetail(bExcludeFirst, iDataLength, CwlbDemoData.Create(_objRandom, iDataLength)));
                }
            }
        }

        /// <summary>
        /// Messung für 1 Domäne im gleichen Prozessraum.
        /// </summary>
        public void Test1DomainInprocess(int iProgress)
        {
            TimeSpan tsStart;
            TimeSpan tsEnd;
            TimeSpan tsDelta;
            int iProgressDelta = 25 / DataLengthCollection.Count;

            // Durchführung der Messung für alle Datengrössen
            foreach (CwlbMeasurementDetail objDetail in MeasurementCaseCollection[InProcess].Measurement)
            {
                tsStart = TimeSpan.FromSeconds(0);
                tsEnd = TimeSpan.FromSeconds(0);
                tsDelta = TimeSpan.FromSeconds(0);
                CwlbDemoData.InstanceCounter = 0;

                for (int iCount = 0; iCount < MAX_LOOP; iCount++)
                {
                    tsStart = _objStopwatch.Elapsed;
                    objDetail.DataObejct = CwlbDemoData.Modify(objDetail.DataObejct);
                    tsEnd = _objStopwatch.Elapsed;
                    objDetail.TimeSpanCollection.Add(tsEnd - tsStart - objDetail.DataObejct.ServerTime);
                }

                // Allgemeine Daten zusammentragen und benachrichtigen
                objDetail.InstanceCounter = MAX_LOOP + CwlbDemoData.InstanceCounter;

                // Fortschritt melden
                iProgress += iProgressDelta;
                Worker.ReportProgress(iProgress, new CwlbModelChangedEventArgs(InProcess, objDetail));
            }

            // Festlegend er serverseitigen Instanzanzahl
            MeasurementCaseCollection[InProcess].ServersideInstanceCount = 0;
        }

        /// <summary>
        /// Messung für 2 Domänen im gleichen Prozessraum.
        /// </summary>
        /// <param name="iProgress"></param>
        public void Test2DomainsInprocess(int iProgress)
        {
            TimeSpan tsStart;
            TimeSpan tsEnd;
            TimeSpan tsDelta;
            int iProgressDelta = 25 / DataLengthCollection.Count;
            AppDomain objAdditionalDomain = null;

            tsStart = TimeSpan.FromSeconds(0);
            tsEnd = TimeSpan.FromSeconds(0);
            tsDelta = TimeSpan.FromSeconds(0);
            CwlbDemoData.InstanceCounter = 0;
            objAdditionalDomain = AppDomain.CreateDomain("WeroSoft.Samples.Measurement");
            CwlbRemoting objRemoting = objAdditionalDomain.CreateInstanceAndUnwrap("WeroSoft.Samples.LearningWcfClient", "WeroSoft.Samples.CwlbRemoting", null) as CwlbRemoting;

            // Durchführung der Messung für alle Datengrössen
            foreach (CwlbMeasurementDetail objDetail in MeasurementCaseCollection[DomainInProcess].Measurement)
            {
                // Durchführung der Messung für alle Datengrössen
                for (int iCount = 0; iCount < MAX_LOOP; iCount++)
                {
                    tsStart = _objStopwatch.Elapsed;
                    objDetail.DataObejct = objRemoting.Modify(objDetail.DataObejct);
                    tsEnd = _objStopwatch.Elapsed;
                    objDetail.TimeSpanCollection.Add(tsEnd - tsStart - objDetail.DataObejct.ServerTime);
                }

                // Allgemeine Daten zusammentragen und benachrichtigen
                objDetail.InstanceCounter = MAX_LOOP + CwlbDemoData.InstanceCounter;

                // Fortschritt melden
                iProgress += iProgressDelta;
                Worker.ReportProgress(iProgress, new CwlbModelChangedEventArgs(DomainInProcess, objDetail));
            }

            // Festlegend er serverseitigen Instanzanzahl
            MeasurementCaseCollection[DomainInProcess].ServersideInstanceCount = objRemoting.GetInstanceCounter();

            // Remoting beenden
            objRemoting = null;
            AppDomain.Unload(objAdditionalDomain);
            objAdditionalDomain = null;
        }

        /// <summary>
        /// Messung für 2 Prozesse auf der gleichen Maschine.
        /// </summary>
        /// <param name="iProgress"></param>
        public void Test1MachineOutOfProcess(int iProgress)
        {
            TimeSpan tsStart;
            TimeSpan tsEnd;
            TimeSpan tsDelta;
            int iProgressDelta = 25 / DataLengthCollection.Count;
            Mutex objMutex = null;
            Process objProcess = null;
            try
            {
                // Mutex für die Serverautomation bereitstellen
                objMutex = new Mutex(true, CwlbPerformanceService.AUTOMATION_MUTEX_NAME);

                // Server starten
                ProcessStartInfo objStartInfo = new ProcessStartInfo("WeroSoft.Samples.LearningWcfServer.exe", CwlbPerformanceService.NAMED_PIPE);
                objStartInfo.WindowStyle = ProcessWindowStyle.Hidden;
                objProcess = Process.Start(objStartInfo);
                // Warten bis der Server gestartet ist
                Thread.Sleep(1000);

                // Eigenen Client konfigurieren
                IwlbPerformanceService objProxy = null;
                NetNamedPipeBinding objBinding = new NetNamedPipeBinding(NetNamedPipeSecurityMode.None);
                objBinding.MaxReceivedMessageSize = int.MaxValue;
                objBinding.ReaderQuotas.MaxArrayLength = int.MaxValue;
                EndpointAddress objEndpointAddress = new EndpointAddress("net.pipe://localhost/LearningWcf");
                objProxy = ChannelFactory<IwlbPerformanceService>.CreateChannel(objBinding, objEndpointAddress);

                // Verbindung prüfen
                string strServerName = objProxy.Ping();
                if (strServerName != Environment.MachineName)
                {
                    return;
                }
                else
                {
                    // Remoting beenden
                    ((IChannel)objProxy).Close();
                    objProxy = null;
                }

                // Durchführung der Messung für alle Datengrössen
                foreach (CwlbMeasurementDetail objDetail in MeasurementCaseCollection[OutOfProcessLocal].Measurement)
                {
                    tsStart = TimeSpan.FromSeconds(0);
                    tsEnd = TimeSpan.FromSeconds(0);
                    tsDelta = TimeSpan.FromSeconds(0);
                    CwlbDemoData.InstanceCounter = 0;
                    objProxy = ChannelFactory<IwlbPerformanceService>.CreateChannel(objBinding, objEndpointAddress);

                    // Durchführung der Messung für alle Datengrössen
                    for (int iCount = 0; iCount < MAX_LOOP; iCount++)
                    {
                        tsStart = _objStopwatch.Elapsed;
                        objDetail.DataObejct = objProxy.Modify(objDetail.DataObejct);
                        tsEnd = _objStopwatch.Elapsed;
                        objDetail.TimeSpanCollection.Add(tsEnd - tsStart - objDetail.DataObejct.ServerTime);
                    }

                    // Allgemeine Daten zusammentragen und benachrichtigen
                    objDetail.InstanceCounter = MAX_LOOP + CwlbDemoData.InstanceCounter;


                    // Festlegend er serverseitigen Instanzanzahl
                    MeasurementCaseCollection[OutOfProcessLocal].ServersideInstanceCount = objProxy.GetInstanceCounter();

                    // Remoting beenden
                    ((IChannel)objProxy).Close();
                    objProxy = null;

                    // Fortschritt melden
                    iProgress += iProgressDelta;
                    Worker.ReportProgress(iProgress, new CwlbModelChangedEventArgs(OutOfProcessLocal, objDetail));
                }
            }
            catch (Exception)
            {
            }
            finally
            {
                // Server die Beendigung signalisieren 
                if (objMutex != null)
                {
                    objMutex.ReleaseMutex();
                }

                // Den Prozess beenden
                if (objProcess != null)
                {
                    objProcess.WaitForExit(10000);
                    if (!objProcess.HasExited)
                    {
                        objProcess.Kill();
                    }
                }
            }
        }

        /// <summary>
        /// Messung für 2 Prozesse auf 2 Maschinen.
        /// </summary>
        /// <param name="iProgress"></param>
        public void Test2MachinesOutOfProcess(int iProgress)
        {
            TimeSpan tsStart;
            TimeSpan tsEnd;
            TimeSpan tsDelta;
            int iProgressDelta = 25 / DataLengthCollection.Count;
            string strRemoteMachineName = RemoteMachineName;
            IwlbPerformanceService objProxy = null;
            IPAddress objAddress = CwlbMeasurementModel.GetIpAddressRemoteMachine(IpAddressRemoteMachine, ref strRemoteMachineName);
            if (IPAddress.Equals(objAddress, IPAddress.None))
            {
                return;
            }
            try
            {
                // Eigenen Client konfigurieren
                NetTcpBinding objBinding = new NetTcpBinding(SecurityMode.None);
                objBinding.ReaderQuotas.MaxArrayLength = int.MaxValue;
                objBinding.MaxReceivedMessageSize = int.MaxValue;
                EndpointAddress objEndpointAddress = new EndpointAddress(string.Format("net.tcp://{0}:5678/LearningWcf", objAddress.ToString()));
                objProxy = ChannelFactory<IwlbPerformanceService>.CreateChannel(objBinding, objEndpointAddress);

                // Verbindung prüfen
                string strServerName = objProxy.Ping();
                if (!RemoteMachineName.ToLower().StartsWith(strServerName.ToLower()))
                {
                    // Remoting beenden
                    ((IChannel)objProxy).Close();
                    objProxy = null;
                    return;
                }

                // Durchführung der Messung für alle Datengrössen
                foreach (CwlbMeasurementDetail objDetail in MeasurementCaseCollection[OutOfProcessRemote].Measurement)
                {
                    tsStart = TimeSpan.FromSeconds(0);
                    tsEnd = TimeSpan.FromSeconds(0);
                    tsDelta = TimeSpan.FromSeconds(0);
                    CwlbDemoData.InstanceCounter = 0;
                    objProxy = ChannelFactory<IwlbPerformanceService>.CreateChannel(objBinding, objEndpointAddress);

                    // Durchführung der Messung für alle Datengrössen
                    for (int iCount = 0; iCount < MAX_LOOP; iCount++)
                    {
                        tsStart = _objStopwatch.Elapsed;
                        objDetail.DataObejct = objProxy.Modify(objDetail.DataObejct);
                        tsEnd = _objStopwatch.Elapsed;
                        objDetail.TimeSpanCollection.Add(tsEnd - tsStart - objDetail.DataObejct.ServerTime);
                    }

                    // Allgemeine Daten zusammentragen und benachrichtigen
                    objDetail.InstanceCounter = MAX_LOOP + CwlbDemoData.InstanceCounter;

                    // Festlegend er serverseitigen Instanzanzahl
                    MeasurementCaseCollection[OutOfProcessRemote].ServersideInstanceCount = objProxy.GetInstanceCounter();

                    // Remoting beenden
                    ((IChannel)objProxy).Close();
                    objProxy = null;

                    // Fortschritt melden
                    iProgress += iProgressDelta;
                    Worker.ReportProgress(iProgress, new CwlbModelChangedEventArgs(OutOfProcessRemote, objDetail));
                }
            }
            catch (Exception)
            {
            }
            finally
            {
                if (objProxy != null && ((IChannel)objProxy).State == CommunicationState.Opened)
                {
                    ((IChannel)objProxy).Close();
                    objProxy = null;
                }
            }
        }

        /// <summary>
        /// Liefert die IP-Adresse einer Maschine.
        /// </summary>
        /// <param name="IpAddressRemoteMachine">Angefragte IP-Adresse.</param>
        /// <param name="RemoteMachineName">Name der Maschine. Ist dieser definiert, wird mit Hilfe dieses Namens die IP-Adresse Priorität aufgelöst.</param>
        /// <returns>Liefert die IP-Adresse einer Maschine, oder None wenn die Adresse nicht gefunden wurde.</returns>
        internal static IPAddress GetIpAddressRemoteMachine(string IpAddressRemoteMachine, ref string RemoteMachineName)
        {
            try
            {
                // Prüfen ob die IP Adresse eingegeben wurde
                if (!string.IsNullOrEmpty(RemoteMachineName))
                {
                    Ping objPing = new Ping();
                    PingReply objReply = objPing.Send(RemoteMachineName);
                    if (objReply.Status == IPStatus.Success)
                    {
                        return objReply.Address;
                    }
                }

                // Prüfen ob die IP Adresse eingegeben wurde
                else if (!string.IsNullOrEmpty(IpAddressRemoteMachine))
                {
                    Ping objPing = new Ping();
                    PingReply objReply = objPing.Send(IpAddressRemoteMachine);
                    if (objReply.Status == IPStatus.Success)
                    {
                        IPHostEntry objEntry = Dns.GetHostEntry(objReply.Address);
                        RemoteMachineName = objEntry.HostName;
                        return objReply.Address;
                    }
                }

                return IPAddress.None;
            }
            catch
            {
                return IPAddress.None;
            }
        }
    }
}
