﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Data;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert die Hauptansicht der Anwendung. Das Objekt dieser Klasse wird dem Kontext des Hauptfensters zugewiesen.
    /// </summary>
    public class CwlbMeasurementView : CwlbViewBase 
    {
        /// <summary>
        /// Hält die Referenz zum Datenmodell der Messungen.
        /// </summary>
        private CwlbMeasurementModel Model;

        /// <summary>
        /// Liefert den Namen der lokalen Maschine.
        /// </summary>
        public string LocalMachineName
        {
            get { return Model.LocalMachineName; }
        }

        /// <summary>
        /// Liefert die IP-Adresse der lokalen Maschine.
        /// </summary>
        public string IpAddressLocalMachine
        {
            get { return Model.IpAddressLocalMachine; }
        }

        /// <summary>
        /// Liefert oder definiert den Namen der entfernten Maschine.
        /// </summary>
        public string IpAddressRemoteMachine
        {
            get { return Model.IpAddressRemoteMachine; }
            set {
                Model.IpAddressRemoteMachine = value;
                OnPropertyChanged("IpAddressRemoteMachine");
            }
        }

        /// <summary>
        /// Liefert oder definiert den Namen der entfernten Maschine.
        /// </summary>
        public string RemoteMachineName
        {
            get { return Model.RemoteMachineName; }
            set {
                Model.RemoteMachineName = value;
                OnPropertyChanged("RemoteMachineName");
            }
        }

        /// <summary>
        /// Liefert die Liste der Übersichtsresultate aller Messreihen.
        /// </summary>
        public ObservableCollection<CwlbOverviewDetailView> Overview { get; private set; }

        /// <summary>
        /// Liefert die Liste der Messreihen für die Messung innerhalb des Prozessraums mit einer Domäne.
        /// </summary>
        public ObservableCollection<CwlbDetailView> InProcess { get; private set; }

        /// <summary>
        /// Liefert die Liste der Messreihen für die Messung von 2 Domänen in einem Prozessraums.
        /// </summary>
        public ObservableCollection<CwlbDetailView> DomainInProcess { get; private set; }

        /// <summary>
        /// Liefert die Liste der Messreihen für die Messung von 2 Prozessen auf einer Maschine.
        /// </summary>
        public ObservableCollection<CwlbDetailView> OutOfProcessLocal { get; private set; }

        /// <summary>
        /// Liefert die Liste der Messreihen für die Messung von 2 Prozessen auf zwei Maschinen.
        /// </summary>
        public ObservableCollection<CwlbDetailView> OutOfProcessRemote { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbMeasurementView.
        /// </summary>
        /// <param name="cwlbMeasurement"></param>
        public CwlbMeasurementView(CwlbMeasurementModel cwlbMeasurement)
        {
            // Wert übernehmen
            Model = cwlbMeasurement;
            Overview = new ObservableCollection<CwlbOverviewDetailView>();
            InProcess = new ObservableCollection<CwlbDetailView>();
            DomainInProcess = new ObservableCollection<CwlbDetailView>();
            OutOfProcessLocal = new ObservableCollection<CwlbDetailView>();
            OutOfProcessRemote = new ObservableCollection<CwlbDetailView>();
        }

        /// <summary>
        /// Löscht die vorhandenen Datenlisten.
        /// </summary>
        internal void Clear()
        {
            Overview.Clear();
            InProcess.Clear();
            DomainInProcess.Clear();
            OutOfProcessLocal.Clear();
            OutOfProcessRemote.Clear();
        }

        /// <summary>
        /// Verarbeitet den gemeldeten Fortschritt aus der Messung.
        /// </summary>
        /// <param name="objEvent">Daten die verarbeitet wurden.</param>
        internal void Progress(CwlbModelChangedEventArgs objEvent)
        {
            // Lesen des Übersichtselements
            CwlbOverviewDetailView objOverviewDetail = CheckOverviewCollection(Overview, objEvent.Detail);
            if (objEvent.MeasurementCase == CwlbMeasurementModel.InProcess)
            {
                // Verarbeiten einer InProcess Messung
                objOverviewDetail.TimeSpanInProcessAvarage = objEvent.Detail.TimeSpanAvarage;
                CheckDetailCollection(InProcess, objEvent.Detail);
                OnPropertyChanged("InProcess");
            }
            else if (objEvent.MeasurementCase == CwlbMeasurementModel.DomainInProcess)
            {
                // Verarbeiten einer DomainInProcess Messung
                objOverviewDetail.TimeSpanDomainInProcessAvarage = objEvent.Detail.TimeSpanAvarage;
                CheckDetailCollection(DomainInProcess, objEvent.Detail);
                OnPropertyChanged("DomainInProcess");
            }
            else if (objEvent.MeasurementCase == CwlbMeasurementModel.OutOfProcessLocal)
            {
                // Verarbeiten einer OutOfProcessLocal Messung
                objOverviewDetail.TimeSpanProcessLocalAvarage = objEvent.Detail.TimeSpanAvarage;
                CheckDetailCollection(OutOfProcessLocal, objEvent.Detail);
                OnPropertyChanged("OutOfProcessLocal");
            }
            else if (objEvent.MeasurementCase == CwlbMeasurementModel.OutOfProcessRemote)
            {
                // Verarbeiten einer OutOfProcessRemote Messung
                objOverviewDetail.TimeSpanProcessRemoteAvarage = objEvent.Detail.TimeSpanAvarage;
                CheckDetailCollection(OutOfProcessRemote, objEvent.Detail);
                OnPropertyChanged("OutOfProcessRemote");
            }
            OnPropertyChanged("Overview");
        }

        /// <summary>
        /// Verarbeitet einer spezifischen Messung in der Übersichtsliste.
        /// </summary>
        /// <param name="objCollection">Liste die verwendet werden soll.</param>
        /// <param name="objDetail">Messreihe, die verarbeitet werden soll.</param>
        /// <returns>Verändertes oder neu erstelltes View-Element.</returns>
        private CwlbOverviewDetailView CheckOverviewCollection(ObservableCollection<CwlbOverviewDetailView> objCollection, CwlbMeasurementDetail objDetail)
        {
            CwlbOverviewDetailView objOverviewDetail = objCollection.FirstOrDefault(item => item.DataLength == objDetail.DataLength);
            if (objOverviewDetail == null)
            {
                objOverviewDetail = new CwlbOverviewDetailView(objDetail.DataLength);
                objCollection.Add(objOverviewDetail);
            }
            return objOverviewDetail;
        }

        /// <summary>
        /// Verarbeitet einer spezifischen Messung in einer der Listen.
        /// </summary>
        /// <param name="objCollection">Liste die verwendet werden soll.</param>
        /// <param name="objDetail">Messreihe, die verarbeitet werden soll.</param>
        /// <returns>Verändertes oder neu erstelltes View-Element.</returns>
        private CwlbDetailView CheckDetailCollection(ObservableCollection<CwlbDetailView> objCollection, CwlbMeasurementDetail objDetail)
        {
            CwlbDetailView objDetailView = objCollection.FirstOrDefault(item => item.Detail.DataLength == objDetail.DataLength);
            if (objDetailView == null)
            {
                objDetailView = new CwlbDetailView(objDetail);
                objCollection.Add(objDetailView);
                OnPropertyChanged("");
            }
            return objDetailView;
        }
    }
}
