﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WeroSoft.Samples.LearningWcfContract;
using System.ServiceModel;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Implementierung des Dienstes mit Rückrufen.
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single, ConcurrencyMode=ConcurrencyMode.Reentrant)]
    public class CwlbCallbackDemoServiceSingle : CwlbDemoServiceBase, IwlbCallbackDemoService
    {
        /// <summary>
        /// Definiert ein Objekt für die interne Synchronisation.
        /// </summary>
        private object _objSyncRoot;

        /// <summary>
        /// Nimmt eine Liste der aktuell registrierten Rückrufe (engl. callbacks) auf
        /// </summary>
        private List<IwlbClientCallback> _cobjCallbacks;

        /// <summary>
        /// Systeminternen Timer.
        /// </summary>
        private System.Timers.Timer _objTimer;

        /// <summary>
        /// Definiert die Länge des Intervalls des Timers.
        /// </summary>
        private double _dInterval;

        /// <summary>
        /// Hält eine Referenz auf den verwendeten Instanzkontext
        /// </summary>
        private static InstanceContext _objContext;

        /// <summary>
        /// Hält eine Referenz auf den verwendeten Instanzkontext
        /// </summary>
        protected override InstanceContext LastInstanceContext 
        {
            get { return _objContext; }
            set { _objContext = value; }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbCallbackDemoServiceSingle.
        /// </summary>
        public CwlbCallbackDemoServiceSingle()
        {
            _objSyncRoot = new object();
            _cobjCallbacks = new List<IwlbClientCallback>();
            _dInterval = 1000.0;
        }

        /// <summary>
        /// Definiert eine einfache Methode für die Demonstration der Polymorphie in Diensten.
        /// </summary>
        /// <param name="strClientData">Daten, die der Client für den Ping an den Service übermittelt.</param>
        /// <returns>Formatierte Antwort des Dienstes.</returns>
        [OperationBehavior(ReleaseInstanceMode = ReleaseInstanceMode.BeforeAndAfterCall)]
        public override CwlbInfoEventArgs DemoCall(string strClientData)
        {
            return DemoCallPrepare(strClientData);
        }

        /// <summary>
        /// Führt einen blockierenden Rückruf aus.
        /// </summary>
        /// <param name="strClientData">Daten, die der Client für den Ping an den Service übermittelt.</param>
        public void BlockingCallback(string strClientData)
        {
            IwlbClientCallback objCallback = OperationContext.Current.GetCallbackChannel<IwlbClientCallback>();
            objCallback.BlockingClientCallback(base.Ping(strClientData));
        }

        /// <summary>
        /// Führt einen nicht blockierenden Rückruf aus.
        /// </summary>
        /// <param name="strClientData">Daten, die der Client für den Ping an den Service übermittelt.</param>
        public void NonBlockingCallback(string strClientData)
        {
            IwlbClientCallback objCallback = OperationContext.Current.GetCallbackChannel<IwlbClientCallback>();
            objCallback.NonBlockingClientCallback(base.Ping(strClientData));
        }

        /// <summary>
        /// Registriert einen Client-Rückrufvertrag für die zyklische Ausführung eines Rückrufs.
        /// </summary>
        public void RegisterClient()
        {
            lock (_objSyncRoot)
            {
                // Registrieren des Rückrufvertrags aus dem Operationskontext der WCF
                _cobjCallbacks.Add(OperationContext.Current.GetCallbackChannel<IwlbClientCallback>());

                // Prüfen, ob der serverseitige Timer gestartet ist; starten wenn notwendig
                if (_objTimer == null)
                {
                    _objTimer = new System.Timers.Timer();
                    _objTimer.Interval = _dInterval;
                    _objTimer.Elapsed += new System.Timers.ElapsedEventHandler(OnTimerElapsed);
                    _objTimer.Start();
                    base.OnServiceObjectAction(new CwlbInfoEventArgs 
                    {
                        Sender = Id.ToString(),
                        Text = "Timer gestartet" 
                    });
                }
            }
        }

        /// <summary>
        /// Abmelden eines registrierten Client für die zyklische Ausführung eines Rückrufs.
        /// </summary>
        public void UnregisterClient()
        {
            lock (_objSyncRoot)
            {
                _cobjCallbacks.Remove(OperationContext.Current.GetCallbackChannel<IwlbClientCallback>());
            }
        }

        /// <summary>
        /// Einstellen des Intervalls für zyklische Rückrufe.
        /// </summary>
        /// <param name="dValue">Grösse des Intervalls in Sekunden</param>
        public void SetInterval(double dValue)
        {
            lock (_objSyncRoot)
            {
                // Wert in Millisekunden umrechnen und sowohl intern speichern
                // als auch für den Timer definieren.
                _objTimer.Interval = _dInterval = dValue * 1000;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Verarbeitung des Timerintervalls.
        /// </summary>
        private void OnTimerElapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            // Bereitstellen einer Liste für die automatische Abmeldung eines Clients,
            // der einen Fehler auslöst
            List<IwlbClientCallback> _cobjFaultedCallbacks = new List<IwlbClientCallback>();
            CwlbInfoEventArgs objEventArgs = new CwlbInfoEventArgs
            {
                Sender = "ZyklischerServerCallback",
                Text = string.Format("Maschine: {0}", Environment.MachineName)
            };

            lock (_objSyncRoot)
            {
                // Durchführen der Rückrufe zu allen registrierten Clients
                foreach (IwlbClientCallback objCallback in _cobjCallbacks)
                {
                    try
                    {
                        objCallback.CyclicClientCallback(objEventArgs.Sender, objEventArgs);
                    }
                    catch
                    {
                        // Den fehlerbehafteten Rückruf aus der Liste herausnehmen
                        _cobjFaultedCallbacks.Add(objCallback);
                        OnServiceObjectAction(new CwlbInfoEventArgs
                        {
                            Sender = this.Id.ToString(),
                            Text = string.Format("Sitzung '{0}' aus dem Rückruf entfernt.", ((IClientChannel)objCallback).SessionId)
                        });
                    }
                }

                // Alle fehlerbehafteten Rückrufe aus der Liste der Rückrufverträge austragen
                foreach (IwlbClientCallback objCallback in _cobjFaultedCallbacks)
                {
                    _cobjCallbacks.Remove(objCallback);
                }

                // Prüfen ob noch weitere Rückrufe notwendig sind, falls nicht, soll der
                // Timer gestoppt werden und der Rückrufmechanismus so ausgeschaltet werden
                if (_cobjCallbacks.Count == 0)
                {
                    _objTimer.Stop();
                    _objTimer.Dispose();
                    _objTimer = null;
                    base.OnServiceObjectAction(new CwlbInfoEventArgs { Sender = Id.ToString(), Text = "Timer gestoppt" });
                }
            }
        }
    }
}
