﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Dispatcher;
using System.ServiceModel;
using WeroSoft.Samples.LearningWcfContract;
using System.ServiceModel.Channels;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Definiert einen eigenen Kontextprovider für die Nutzung einer geteilten Instanziierung.
    /// </summary>
    public class CwlbContextProvider : IInstanceContextProvider
    {
        // Verzeichnis der erzeugten Instanzen. Die Instanzen in diesem Verzeichnis
        // können von mehreren Sitzungen genutzt werden
        private Dictionary<String, CwlbContextInfo> _cobjContextCollection = new Dictionary<string, CwlbContextInfo>();

        /// <summary>
        /// Steuert die Nutzung der Instanz, nachdem eine neue Nachricht empfangen wurde.
        /// </summary>
        /// <param name="objMessage">Die Nachricht, die empfangen wurde.</param>
        /// <param name="objChannel">Der Kanal, der die Nachricht empfangen hat.</param>
        /// <returns>
        /// Das InstanceContext-Objekt, das zur Verarbeitung der Nachricht genutzt wird, 
        /// respektive null, um ein neues InstanceContext-Objekt zu erstellen.
        /// </returns>
        public InstanceContext GetExistingInstanceContext(Message objMessage, IContextChannel objChannel)
        {
            // Lokale Daten
            string strInstanceId = String.Empty;
            bool bHasSession = false;
            CwlbContextInfo objContextInfo = null;

            // Suchen des Headereintrags für die ID des Dienstobejkts das benutzt werden soll.
            int iIndex = objMessage.Headers.FindHeader(GwlbContractGlobals.SharedHeaderName, GwlbContractGlobals.SharedHeaderNamespace);
            if (iIndex != -1)
            {
                strInstanceId = objMessage.Headers.GetHeader<string>(iIndex);
            }

            // Prüfen on eine Sitzung vorhanden ist
            if (objChannel.SessionId != null)
            {
                bHasSession = true;
                objContextInfo = objChannel.Extensions.Find<CwlbContextInfo>();
            }

            // Wenn keine Instanz identifikation angegeben wurde oder das Instanz objekt nicht gefunden werden konnte
            // wird eine neue Instanz des Dienstes erzeugt.
            if ((strInstanceId == null) || !this._cobjContextCollection.TryGetValue(strInstanceId, out objContextInfo))
            {
                objContextInfo = new CwlbContextInfo(strInstanceId);

                // Die neue Instanz registrieren
                this._cobjContextCollection.Add(strInstanceId, objContextInfo);
                if (bHasSession)
                {
                    objChannel.Extensions.Add(objContextInfo);
                }
                return null;
            }
            else
            {
                // Es ist eine Instanz vorhanden, diese verwenden.
                InstanceContext instanceContext = objContextInfo.Instance;
                if (bHasSession)
                {
                    instanceContext.IncomingChannels.Add(objChannel);
                }
                return instanceContext;
            }
        }

        /// <summary>
        /// Wird aufgerufen, wenn von der Methode GetExistingInstanceContext null zurückgegeben wird.
        /// </summary>
        /// <param name="objInstanceContext">Das neue Instanzobjekt.</param>
        /// <param name="objMessage">Die Nachricht, die empfangen wurde.</param>
        /// <param name="objChannel">Der Kanal, der die Nachricht empfangen hat.</param>
        public void InitializeInstanceContext(InstanceContext objInstanceContext, Message objMessage, IContextChannel objChannel)
        {
            CwlbContextInfo objContextInfo = null;

            // Pürfen ob eine Sitzung vorhanden ist
            if (objChannel.SessionId != null)
            {
                // Es ist eine Sitzung vorhanden
                objInstanceContext.IncomingChannels.Add(objChannel);
                objContextInfo = objChannel.Extensions.Find<CwlbContextInfo>();
            }
            else
            {
                // Es ist keine Sitzung vorhanden
                int headerIndex = objMessage.Headers.FindHeader(GwlbContractGlobals.SharedHeaderName, GwlbContractGlobals.SharedHeaderNamespace);
                if (headerIndex != -1)
                {
                    string strInstanceId = objMessage.Headers.GetHeader<string>(headerIndex);
                    if (strInstanceId != null)
                    {
                        _cobjContextCollection.TryGetValue(strInstanceId, out objContextInfo);
                    }
                }
            }

            // Prüfen ob das Kontext Informationsobjekte bekannt ist und Kontext aus Parameterlist dem Informationsobjekt zuweisen
            if (objContextInfo != null)
            {
                objContextInfo.Instance = objInstanceContext;
            }
        }

        /// <summary>
        /// Wird aufgerufen, wenn alle InstanceContext-Aktivitäten die Aktivierung 
        /// der Implementierungen abgeschlossen haben, um zu verhindern, dass 
        /// InstanceContext wiederverwendet wird.
        /// </summary>
        /// <param name="objInstanceContext">Das InstanceContext-Objekt, das als inaktiv betrachtet wird.</param>
        /// <returns>
        /// True, wenn das InstanceContext-Objekt schließen und den Wert von ServiceThrottle.MaxConcurrentInstances 
        /// angepasst werden soll; andernfalls false.
        /// </returns>
        public bool IsIdle(System.ServiceModel.InstanceContext objInstanceContext)
        {
            return false;
        }

        /// <summary>
        /// Wird aufgerufen, wenn die IsIdle-Methode false zurückgibt, damit Implementierungen einen 
        /// Rückruf durchführen, um zu benachrichtigen, wenn das InstanceContext-Objekt geschlossen wird.
        /// </summary>
        /// <param name="objCallback">
        /// Das InstanceContextIdleCallback-Objekt, das benachrichtigt, wenn das InstanceContext-Objekt 
        /// geschlossen und wiederverwendet werden soll.
        /// </param>
        /// <param name="objInstanceContext">Das InstanceContext-Objekt, das als inaktiv betrachtet wird.</param>
        public void NotifyIdle(InstanceContextIdleCallback objCallback, InstanceContext objInstanceContext)
        {
            // Keine Implementierung
        }
    }
}
