﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WeroSoft.Samples.LearningWcfContract;
using System.ServiceModel;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Basisklasse für die Implementierung der Dienste
    /// </summary>
    [ServiceBehavior(IncludeExceptionDetailInFaults=true)]
    public abstract class CwlbDemoServiceBase : IwlbDemoService, IDisposable
    {
        /// <summary>
        /// Ereignis für die einfachere Vermittlung von Spezialsituationen der Dienstobjekte.
        /// </summary>
        public static event EventHandler<CwlbInfoEventArgs> ServiceObjectAction = delegate { };

        /// <summary>
        /// Liefert die Objektidentifikation des Dienstobjekts.
        /// </summary>
        public Guid Id { get; set; }

        /// <summary>
        /// Hält eine Referenz auf den verwendeten Instanzkontext
        /// </summary>
        protected abstract InstanceContext LastInstanceContext { get; set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbDemoServiceBase.
        /// </summary>
        public CwlbDemoServiceBase()
        {
            Id = Guid.NewGuid();
            
            OnServiceObjectAction(new CwlbInfoEventArgs 
            {
                Sender = Id.ToString(),
                Text = string.Format(".ctor Type: {0}", 
                   this.GetType().Name)
            });
        }

        /// <summary>
        /// Freigabe der Instanz.
        /// </summary>
        public void Dispose()
        {
            OnServiceObjectAction(new CwlbInfoEventArgs { Text = "Dispose type " + this.GetType().Name });
        }

        /// <summary>
        /// Dienstmethode eines applikatorischen Pings.
        /// </summary>
        /// <param name="strClientData">Daten des Clients.</param>
        /// <returns>Formatierte Antwort des Dienstes.</returns>
        public CwlbInfoEventArgs Ping(string strClientData)
        {
            LastInstanceContext = OperationContext.Current.InstanceContext;
            return new CwlbInfoEventArgs
            {
                Sender = Id.ToString(),
                Text = string.Format("Maschine: {0}, Dienstinstanziierung: {1}",
                    Environment.MachineName,
                    GetInstanceContextMode())
            };
        }

        /// <summary>
        /// Definiert eine einfache Methode für die Demonstration der Polymorphie in Diensten.
        /// </summary>
        /// <param name="strClientData">Daten, die der Client für den Ping an den Service übermittelt.</param>
        /// <returns>Formatierte Antwort des Dienstes.</returns>
        public abstract CwlbInfoEventArgs DemoCall(string strClientData);


        /// <summary>
        /// Löst das Aktionsereignis der Klasse aus.
        /// </summary>
        /// <param name="objEventArgs">Argument, das mit dem Ereignis weitergeleitet werden soll.</param>
        protected void OnServiceObjectAction(CwlbInfoEventArgs objEventArgs)
        {
            ServiceObjectAction(this, objEventArgs);
        }

        /// <summary>
        /// Erstellt die eigentliche Nachricht für den Aufruf der Methode DemoCall.
        /// </summary>
        /// <param name="strClientData">Daten, die der Client für den Ping an den Service übermittelt.</param>
        /// <returns>Formatierte Antwort des Dienstes.</returns>
        protected CwlbInfoEventArgs DemoCallPrepare(string strClientData)
        {
            CwlbInfoEventArgs objEvent = Ping(strClientData);
            InstanceContext objContext = OperationContext.Current.InstanceContext;

            objEvent.Text = string.Format(
                "{0} Kontext: DemoCall in: {1}",
                object.ReferenceEquals(LastInstanceContext, objContext) ? "Gleicher" : "Unterschiedlicher",
                GetType().Name);

            LastInstanceContext = objContext;

            return objEvent;
        }

        /// <summary>
        /// Liefert den InstanceContextMode der Dienstinstanz.
        /// </summary>
        /// <returns></returns>
        private string GetInstanceContextMode()
        {
            // Zuerst prüfen ob eine Shared Implementierung vorlierg
            CwlbShareableServiceAttribute objSharedAttribute = OperationContext.Current.Host.Description.Behaviors.FirstOrDefault(item => item.GetType() == typeof(CwlbShareableServiceAttribute)) as CwlbShareableServiceAttribute;
            if (objSharedAttribute != null)
            {
                return "Shared";
            }

            // Sonstige Implementierungen 
            ServiceBehaviorAttribute objContextAttribute = OperationContext.Current.Host.Description.Behaviors.FirstOrDefault(item => item.GetType() == typeof(ServiceBehaviorAttribute)) as ServiceBehaviorAttribute;
            if (objContextAttribute != null)
            {
                return objContextAttribute.InstanceContextMode.ToString();
            }
            else
            {
                // Standardwert zurückliefern
                return InstanceContextMode.PerSession.ToString();
            }
        }
    }
}
