﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.Text;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Attribut für die einfache, imperative Registrierung der allgemeinen Fehlerbehandlung.
    /// </summary>
    public sealed class CwlbErrorHandlerAttribute : Attribute, IServiceBehavior
    {
        /// <summary>
        /// Liefert den Typ der Fehlerbehandlung.
        /// </summary>
        public Type ErrorHandlerType { get; private set; }

        /// <summary>
        /// Liefert oder definiert einen Konverter für ein Fehlerdetail.
        /// </summary>
        public Type FaultDetailConverter { get; set; }

        /// <summary>
        /// Liefert oder definiert einen Wert der angibt ob eine Sitzung beim Auftreten eines Fehlers erhalten bleibt oder nicht.
        /// </summary>
        /// <remarks>Der Standardwert dieser Eigenschaft ist true.</remarks>
        public bool KeepSessionAlive { get; set; }

        /// <summary>
        /// Initialisiert eine neue Instanz des Typs CwlbCustomServiceBehaviorAttribute.
        /// </summary>
        /// <param name="objTypeErrorHandler">Typ des ErrorHandlers für diesen Dienst.</param>
        public CwlbErrorHandlerAttribute(Type objTypeErrorHandler)
        {
            ErrorHandlerType = objTypeErrorHandler;
            KeepSessionAlive = true;
        }

        #region IServiceBehavior Members
        /// <summary>
        /// Bietet die Möglichkeit, benutzerdefinierte Daten an Bindungselemente zu übergeben,
        /// um die Vertragsimplementierung zu unterstützen.
        /// </summary>
        /// <param name="objServiceDescription">Die Beschreibung des Diensts.</param>
        /// <param name="objServiceHostBase">Der Host des Dienstes.</param>
        /// <param name="cobjEndpoints">Liste der Dienstpunkte.</param>
        /// <param name="cobjBindingParameters">Benutzerdefinierte Objekte, auf die Bindungselemente Zugriff haben.</param>
        public void AddBindingParameters(ServiceDescription objServiceDescription,
            ServiceHostBase objServiceHostBase, Collection<ServiceEndpoint> cobjEndpoints,
            BindingParameterCollection cobjBindingParameters)
        {
            // Verwenden Sie die Methode AddBindingParameters(), um benutzerdefinierte Datenobjekte der
            // Parameters-Auflistung hinzuzufügen, mit der Bindungselemente zusätzliche Informationen 
            // zur Unterstützung des Vertrags abrufen können.

            // Diese Methode wird einmal für jeden abgehörten URI aufgerufen. Wenn der Dienst beispielsweise 
            // vier Endpunkte hat und zwei über den gleichen abgehörten URI verfügen, wird diese Methode drei 
            // Mal aufgerufen. Der Grund dafür ist, dass die cobjBindingParameters jeden Kanalstapel als Ziel 
            // haben und ein Kanalstapel für jeden abgehörten URI besteht. Da alle Aufrufe diese Endpunkte an
            // diesem abgehörten URI empfangen, rufen zwei Aufrufe jeweils einen ServiceEndpoint ab, und ein
            // Aufruf ruft eine Auflistung von zwei ServiceEndpoint-Objekten ab.
        }

        /// <summary>
        /// Erlaubt die Registrierung von Erweiterungen eines Dispatchers.
        /// </summary>
        /// <param name="objServiceDescription">Die Dienstbeschreibung.</param>
        /// <param name="objServiceHostBase">Der Host der gerade erstellt wird.</param>
        public void ApplyDispatchBehavior(ServiceDescription objServiceDescription,
            ServiceHostBase objServiceHostBase)
        {
            foreach (ChannelDispatcherBase objChannelDispatcherBase in objServiceHostBase.ChannelDispatchers)
            {
                ChannelDispatcher objChannelDispatcher = objChannelDispatcherBase as ChannelDispatcher;
                if (objChannelDispatcher == null)
                {
                    continue;
                }

                // Erstellen des Handlers und registrieren desselben
                objChannelDispatcher.ErrorHandlers.Add(
                    Activator.CreateInstance(
                        ErrorHandlerType, 
                        new object[] {FaultDetailConverter,  KeepSessionAlive}) as IErrorHandler);
            }
        }

        /// <summary>
        /// Erlaubt es, in den Prüfprozess einzugreifen, der dazu dient sicherzustellen, dass der Dienst erfolgreich ausgeführt wird.
        /// </summary>
        /// <param name="objServiceDescription">Die Dienstbeschreibung.</param>
        /// <param name="objServiceHostBase">Der Host der gerade erstellt wird.</param>
        public void Validate(ServiceDescription objServiceDescription, ServiceHostBase objServiceHostBase)
        {
            // Hier können Sie eine beliebige Funktionalität einbringen. Lösen Sie eine Ausnahme aus,
            // wenn Sie feststellen, dass etwas nicht in Ordnung ist.
        }
        #endregion  
    }
}
