﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.Text;
using WeroSoft.Samples.LearningWcfContract;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Definiert eine eigene Fehlerbehandlung für einen Dienst
    /// </summary>
    public sealed class CwlbServiceErrorHandler : IErrorHandler
    {
        /// <summary>
        /// Liefert den Konverter für einen Fehler.
        /// </summary>
        public CwlbFaultConverter FaultConverter { get; private set; }

        /// <summary>
        /// Liefert einen Wert der Angibt, ob eine Sitzung in den Fehlerzustand wechselt oder nicht.
        /// </summary>
        public bool KeepSessionAlive { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbServiceErrorHandler.
        /// </summary>
        public CwlbServiceErrorHandler()
            : this(typeof(CwlbFaultConverter), true)
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbServiceErrorHandler.
        /// </summary>
        /// <param name="objTypeFaultConverter">Typ des Konverters, der für Fehlerdetails verwendet werden soll.</param>
        public CwlbServiceErrorHandler(Type objTypeFaultConverter)
            : this(objTypeFaultConverter, true)
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbServiceErrorHandler.
        /// </summary>
        /// <param name="objTypeFaultConverter">Typ des Konverters, der für Fehlerdetails verwendet werden soll.</param>
        /// <param name="bKeepSessionAlive">Definiert ob die Sitzung erhalten beliben soll.</param>
        public CwlbServiceErrorHandler(Type objTypeFaultConverter, bool bKeepSessionAlive)
        {
            KeepSessionAlive = bKeepSessionAlive;
            FaultConverter = Activator.CreateInstance(objTypeFaultConverter) as CwlbFaultConverter;
        }

        /// <summary>
        /// Eigentliche Fehlerbehandlung.
        /// </summary>
        /// <param name="objException">Aktuelles Fehlerobjekt.</param>
        /// <returns>
        /// True, wenn die Sitzung nicht abgebrochen werden soll (wenn eine Sitzung vorhanden ist) und der Instanzkontext nicht Single ist, andernfalls false.
        /// </returns>
        public bool HandleError(Exception objException)
        {
            return KeepSessionAlive;
        }

        /// <summary>
        /// Erlaubt es, die ausgelieferte Meldung des Fehlers zu beeinflussen.
        /// </summary>
        /// <param name="objException">Aktuelles Fehlerobjekt</param>
        /// <param name="objSoapVersion">SOAP Version der Mitteilung.</param>
        /// <param name="objFault">Fehlerobjekt, das an den Client zurückgesendet werden wird.</param>
        public void ProvideFault(Exception objException, System.ServiceModel.Channels.MessageVersion objSoapVersion, ref System.ServiceModel.Channels.Message objFault)
        {
            // Prüfen ob die Ausnahme bereits als Fault definiert ist
            if (objException is FaultException)
            {
                return;
            }

            // Prüfen ob ein Converter vorhanden ist
            IwlbFaultDetail objFaultDetail;
            if (FaultConverter != null)
            {
                // Erstellen eines Fehlerdetails
                objFaultDetail = FaultConverter.Convert(objException, OperationContext.Current.Host.Description);
            }
            else
            {
                // Erstellen eines Fehlerdetails
                objFaultDetail = new CwlbFaultDetail
                {
                    MachineName = Environment.MachineName,
                    ServiceName = OperationContext.Current.Host.Description.ServiceType.Name,
                    MethodName = "n.a."
                };
            }

            // Erstellen des Fehlers
            Type objFaultExceptionType = typeof(FaultException<>).MakeGenericType(objFaultDetail.GetType());
            FaultReason objFaultReason = new FaultReason(objException.Message);
            FaultCode objFaultCode = FaultCode.CreateReceiverFaultCode(new FaultCode("0"));
            FaultException objFaultException = (FaultException)Activator.CreateInstance(
                objFaultExceptionType, objFaultDetail, objFaultReason, objFaultCode);
            MessageFault objFaultMessage = objFaultException.CreateMessageFault();
            objFault = Message.CreateMessage(objSoapVersion, objFaultMessage, objFaultException.Action);
        }
    }
}
