﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using WeroSoft.Samples.LearningWcfImplementation;
using WeroSoft.Samples.LearningWcfContract;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;

namespace WeroSoft.Samples.LearningWcfImperativeHost
{
    /// <summary>
    /// Programm für die Demo eines imperativen Hostings.
    /// </summary>
    class Program
    {
        // Interne Konstanten
        private const string NET_TCP = "net.tcp://localhost:4711/";
        private const string HTTP = "http://localhost:4712/";
        private const string TCP_SINGLE = "TcpSingle";
        private const string TCP_PER_SESSION = "TcpPerSession";
        private const string TCP_PER_CALL = "TcpPerCall";
        private const string TCP_FAULT_WITH_DETAILS = "TcpFaultWithDetails";
        private const string TCP_FAULT_WITHOUT_DETAILS = "TcpFaultWithoutDetails";
        private const string TCP_CALLBACK = "TcpCallback";
        private const string SHARED = "Shared";
        private const string HTTP_SINGLE = "HttpSingle";
        private const string HTTP_PER_SESSION = "HttpPerSession";
        private const string HTTP_PER_CALL = "HttpPerCall";

        // Liste der Diensthostinstanzen
        private static List<ServiceHost> _cobjServiceHosts = new List<ServiceHost>();

        /// <summary>
        /// Einsprungpunkt in das Programm.
        /// </summary>
        static void Main()
        {
            Register(typeof(CwlbDemoServicePerCall), typeof(IwlbDemoService), 
                new string[] { NET_TCP + TCP_PER_CALL, HTTP + HTTP_PER_CALL});
            Register(typeof(CwlbDemoServicePerSession), typeof(IwlbDemoService), 
                new string[] { NET_TCP + TCP_PER_SESSION, HTTP + HTTP_PER_SESSION});
            Register(typeof(CwlbDemoServiceSingle), typeof(IwlbDemoService),
                new string[] {NET_TCP + TCP_SINGLE, HTTP + HTTP_SINGLE});
            Register(typeof(CwlbDemoServiceSingle), typeof(IwlbFaultDemoService),
                new string[] { NET_TCP + TCP_FAULT_WITHOUT_DETAILS});
            Register(typeof(CwlbDemoServiceShared), typeof(IwlbDemoService),
                new string[] { NET_TCP + SHARED});
            Register(typeof(CwlbCallbackDemoServiceSingle), typeof(IwlbCallbackDemoService),
                new string[] { NET_TCP + TCP_CALLBACK});
            Register(typeof(CwlbDemoServiceIncludeExceptionInFaultsSingle), typeof(IwlbFaultDemoService),
                new string[] { NET_TCP + TCP_FAULT_WITH_DETAILS});

            // Verhindern das Prozess beendet wird.
            Console.WriteLine("Service Host ist gestartet. Mit einem Tastendruck kann er beendet werden ...");
            Console.ReadKey(true);

            // Schliessen der Hosts
            CloseHosts();
        }

        /// <summary>
        /// Erstellt, konfiguriert und registriert die Diensthostinstanz.
        /// </summary>
        /// <param name="objServiceType">Typ des Dienstes, der die Diensthostinstanz kontrollieren soll.</param>
        /// <param name="objServiceContract">Typ des Dienstvertrags, der mit dem Endpunkt verwendet werden soll.</param>
        /// <param name="astrAddress">Array von Adressen, für die ein Endpunkt erstellt werden soll.</param>
        private static void Register(Type objServiceType, Type objServiceContract, string[] astrAddress)
        {
            // Definiert den Service Host.
            ServiceHost objHost = null;

            try
            {
                // Erstellen der Diensthostinstanz
                objHost = new ServiceHost(objServiceType);

                // Bindung und Endpunkt erstellen und registrieren
                foreach (string strAddress in astrAddress)
                {
                    Binding binding = GetBinding(strAddress);
                    objHost.AddServiceEndpoint(objServiceContract, binding, strAddress);
                }

                // Drosselungen definieren
                ServiceThrottlingBehavior objThrottling = new ServiceThrottlingBehavior();
                objThrottling.MaxConcurrentCalls = 1;
                objThrottling.MaxConcurrentSessions = 20;
                objThrottling.MaxConcurrentInstances = 20;
                objHost.Description.Behaviors.Add(objThrottling);

                // Ereignisse registrieren
                objHost.Opening += new EventHandler(Host_StatusChanged);
                objHost.Opened += new EventHandler(Host_StatusChanged);
                objHost.Closing += new EventHandler(Host_StatusChanged);
                objHost.Closed += new EventHandler(Host_StatusChanged);
                objHost.Faulted += new EventHandler(Host_StatusChanged);
                objHost.UnknownMessageReceived += new EventHandler<UnknownMessageReceivedEventArgs>(Host_UnknownMessageReceived);

                // Diensthost starten und in Liste ergänzen
                objHost.Open();
                _cobjServiceHosts.Add(objHost);
            }
            catch (Exception excObject)
            {
                // Ausnahmebehandlung
                Console.WriteLine(excObject.Message);
            }
        }

        /// <summary>
        /// Schliesst die Diensthostinstanzen.
        /// </summary>
        private static void CloseHosts()
        {
            foreach (ServiceHost objHost in _cobjServiceHosts)
            {
                // Beenden des Dienstes im Normalfall
                if (objHost != null && objHost.State == CommunicationState.Opened)
                {
                    objHost.Close();
                }
                // Beenden des Dienstes im Ausnahmefall
                if (objHost != null && objHost.State == CommunicationState.Faulted)
                {
                    objHost.Abort();
                }
            }
        }

        /// <summary>
        /// Bindung erstellen.
        /// </summary>
        /// <param name="strAddress">Adresse für die eine Bindung erstellt werden soll.</param>
        /// <returns></returns>
        private static Binding GetBinding(string strAddress)
        {
            if (strAddress.StartsWith("net.tcp:") && strAddress.EndsWith(SHARED))
            {
                NetTcpBinding objBinding = new NetTcpBinding();
                objBinding.ReliableSession.Enabled = true;
                return objBinding;
            }
            else if (strAddress.StartsWith("net.tcp:"))
            {
                return new NetTcpBinding();
            }
            else if (strAddress.StartsWith("http:"))
            {
                return new BasicHttpBinding();
            }
            else
            {
                throw new NotSupportedException("Die verlangte Bindung ist in dieser Demo nicht unterstützt.");
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Statusänderung der Diensthostinstanz.
        /// </summary>
        static void Host_StatusChanged(object sender, EventArgs e)
        {
            ServiceHost objHost = sender as ServiceHost;
            if (objHost == null)
            {
                return;
            }

            Console.WriteLine("Dienstinstanz '{0}' hat nun Status '{1}'", objHost.Description.Name, objHost.State);
            switch (objHost.State)
            {
                case CommunicationState.Opened:
                    ShowDetails(objHost);
                    Console.WriteLine(new string('-', 65));
                    break;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Empfang einer inkorrekten Nachricht.
        /// </summary>
        static void Host_UnknownMessageReceived(object sender, UnknownMessageReceivedEventArgs e)
        {
            Console.WriteLine(e.Message);
        }

        /// <summary>
        /// Zeigt Details der Konfiguration der Diensthostinstanz an.
        /// </summary>
        /// <param name="objHost"></param>
        private static void ShowDetails(ServiceHost objHost)
        {
            Console.WriteLine();
            Console.WriteLine("- Angaben zum Dienst");
            Console.WriteLine("    > Dienstname:. . . . . {0}", objHost.Description.Name);
            Console.WriteLine("    > Namensraum:. . . . . {0}", objHost.Description.Namespace);
            Console.WriteLine("    > Diensttyp: . . . . . {0}", objHost.Description.ServiceType);

            foreach (ServiceEndpoint objEndpoint in objHost.Description.Endpoints)
            {
                Console.WriteLine();
                Console.WriteLine("- Endpunktadresse: . . . . {0}", objEndpoint.Address);
                Console.WriteLine("    > Name:. . . . . . . . {0}", objEndpoint.Name);
                Console.WriteLine("    > Vertrag: . . . . . . {0}", objEndpoint.Contract);
                Console.WriteLine("    > IsSystemEndpoint:. . {0}", objEndpoint.IsSystemEndpoint);
                Console.WriteLine("    > ListenUriMode: . . . {0}", objEndpoint.ListenUriMode);
            }
        }
    }
}
