﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Threading;

namespace WeroSoft.Samples.LearningWcfClientUI
{
    /// <summary>
    /// Basisklasse für die Implementierung eines wiederherstellbaren WCF Clients.
    /// </summary>
    /// <typeparam name="T">Type der Schnittstelle des Dienstes, die mit diesem Client benutzt wird.</typeparam>
    public class CwlbClientBase<T> where T: class
    {
        /// <summary>
        /// Liefert oder definiert die Bindung, die mit diesen Client benutzt wird.
        /// </summary>
        public Binding Binding { get; set; }

        /// <summary>
        /// Liefert oder definiert die Endpunktadresse, die mit diesem Client benutzt wird.
        /// </summary>
        public EndpointAddress EndpointAddress { get; set; }

        /// <summary>
        /// Liefert das Proxyobjekt des Dienstes.
        /// </summary>
        public T Proxy { get; private set; }

        /// <summary>
        /// Liefert oder definiert den Kanal, der zum Dienst verwendet wird.
        /// </summary>
        internal IClientChannel Channel
        {
            get
            {
                return Proxy as IClientChannel;
            }
        }

        /// <summary>
        /// Liefert oder definiert den Dispatcher, der für diesen Client benutzt wird.
        /// </summary>
        /// <remarks>
        /// Wenn diese Eigenschaft null ist, erfolgt bei Rückrufen kein Threadwechsel.
        /// </remarks>
        public Dispatcher ClientUIDispatcher { get; set; }

        /// <summary>
        /// Initialisiert eine neue Instanz des Typs CwlbClientBase.
        /// </summary>
        /// <param name="objBinding">Bindung, die der Client verwendet.</param>
        /// <param name="objEndpointAddress">Endpunktadresse, die der Client verwendet.</param>
        /// <param name="objDispatcher">Dispatcher, der bei einer Integration in einen WPF-Client verwendet wird.</param>
        public CwlbClientBase(Binding objBinding, EndpointAddress objEndpointAddress, Dispatcher objDispatcher = null)
        {
            Binding = objBinding;
            EndpointAddress = objEndpointAddress;
            ClientUIDispatcher = objDispatcher;
        }

        /// <summary>
        /// Stellt eine Verbindung zum Dienst wieder her.
        /// </summary>
        protected virtual void Reconnect()
        {
            try
            {
                if (Proxy == null)
                {
                    Proxy = CreateProxy();
                }
                else
                {
                    if (Channel.State == CommunicationState.Opened)
                    {
                        return;
                    }
                    else if (Channel.State == CommunicationState.Faulted)
                    {
                        Channel.Abort();
                        Proxy = CreateProxy();
                    }
                }
            }
            catch
            {
                throw new ApplicationException(Properties.Resources.ServerNotAvailable);
            }
        }

        /// <summary>
        /// Überprüft den Kanalstatus und stellt den Kanal wieder her, wenn er nicht offen ist.
        /// </summary>
        protected void CheckChannel()
        {
            if (Channel.State != CommunicationState.Opened)
            {
                Reconnect();
            }
        }

        /// <summary>
        /// Erstell das eigentliche Proxyobjekt.
        /// </summary>
        /// <returns>Liefert das Proxyobjekt.</returns>
        protected virtual T CreateProxy()
        {
            return ChannelFactory<T>.CreateChannel(Binding, EndpointAddress);
        }

        /// <summary>
        /// Führt eine serverseitige Methode ohne Rückgabewert und ohne Parameter aus.
        /// </summary>
        /// <param name="invoke">Methode, die auszuführen ist.</param>
        protected void ExecuteAction(Action invoke)
        {
            try
            {
                invoke();
            }
            catch (CommunicationException)
            {
                throw new ApplicationException(Properties.Resources.ServerNotAvailable);
            }
        }

        /// <summary>
        /// Führt eine serverseitige Methode ohne Rückgabewert mit einem Parameter aus.
        /// </summary>
        /// <typeparam name="TParam">Typ des Parameters.</typeparam>
        /// <param name="objInvoke">Methode, die auszuführen ist.</param>
        /// <param name="objParam">Parameter für den Methodenaufruf.</param>
        protected void ExecuteAction<TParam>(Action<TParam> objInvoke, TParam objParam)
        {
            try
            {
                objInvoke(objParam);
            }
            catch (CommunicationException)
            {
                throw new ApplicationException(Properties.Resources.ServerNotAvailable);
            }
        }

        /// <summary>
        /// Führt eine serverseitige Methode mit Rückgabewert und ohne Parameter aus.
        /// </summary>
        /// <typeparam name="TResult">Typ des Resultats vom Server.</typeparam>
        /// <param name="objInvoke">Methode, die auszuführen ist.</param>
        /// <returns>Rückgabeobjekt vom Server.</returns>
        protected TResult ExecuteFunc<TResult>(Func<TResult> objInvoke)
        {
            try
            {
                return objInvoke();
            }
            catch (CommunicationException)
            {
                throw new ApplicationException(Properties.Resources.ServerNotAvailable);
            }
        }

        /// <summary>
        /// Führt eine serverseitige Methode mit Rückgabewert und einem Parameter aus.
        /// </summary>
        /// <typeparam name="TParam">Typ des Parameters.</typeparam>
        /// <typeparam name="TResult">Typ des Resultats vom Server.</typeparam>
        /// <param name="objInvoke">Methode, die auszuführen ist.</param>
        /// <param name="objParam">Parameterdaten.</param>
        /// <returns>Rückgabeobjekt vom Server.</returns>
        protected TResult ExecuteFunc<TParam, TResult>(Func<TParam, TResult> objInvoke, TParam objParam)
        {
            try
            {
                return objInvoke(objParam);
            }
            catch (CommunicationException)
            {
                throw new ApplicationException(Properties.Resources.ServerNotAvailable);
            }
        }

        /// <summary>
        /// Führt eine serverseitige Methode mit Rückgabewert und zwei Parameter aus.
        /// </summary>
        /// <typeparam name="TParam1">Typ des ersten Parameters.</typeparam>
        /// <typeparam name="TParam2">Typ des zweiten Parameters.</typeparam>
        /// <typeparam name="TResult">Typ des Resultats vom Server.</typeparam>
        /// <param name="objInvoke">Methode, die auszuführen ist.</param>
        /// <param name="objParam1">Erster Parameter.</param>
        /// <param name="objParam2">Zweiter Parameter.</param>
        /// <returns>Rückgabeobjekt vom Server.</returns>
        protected TResult ExecuteFunc<TParam1, TParam2, TResult>(Func<TParam1, TParam2, TResult> objInvoke, TParam1 objParam1, TParam2 objParam2)
        {
            try
            {
                return objInvoke(objParam1, objParam2);
            }
            catch (CommunicationException)
            {
                throw new ApplicationException(Properties.Resources.ServerNotAvailable);
            }
        }

        /// <summary>
        /// Führt eine serverseitige Methode mit Rückgabewert und vier Parameter aus.
        /// </summary>
        /// <typeparam name="TParam1">Typ des ersten Parameters.</typeparam>
        /// <typeparam name="TParam2">Typ des zweiten Parameters.</typeparam>
        /// <typeparam name="TParam3">Typ des dritten Parameters.</typeparam>
        /// <typeparam name="TResult">Typ des Resultats vom Server.</typeparam>
        /// <param name="objInvoke">Methode, die auszuführen ist.</param>
        /// <param name="objParam1">Erster Parameter.</param>
        /// <param name="objParam2">Zweiter Parameter.</param>
        /// <param name="objParam3">Dritter Parameter.</param>
        /// <returns>Rückgabeobjekt vom Server.</returns>
        protected TResult ExecuteFunc<TParam1, TParam2, TParam3, TResult>(
            Func<TParam1, TParam2, TParam3, TResult> objInvoke, TParam1 objParam1, TParam2 objParam2, TParam3 objParam3)
        {
            try
            {
                return objInvoke(objParam1, objParam2, objParam3);
            }
            catch (CommunicationException)
            {
                throw new ApplicationException(Properties.Resources.ServerNotAvailable);
            }
        }

        /// <summary>
        /// Führt eine serverseitige Methode mit Rückgabewert und vier Parameter aus.
        /// </summary>
        /// <typeparam name="TParam1">Typ des ersten Parameters.</typeparam>
        /// <typeparam name="TParam2">Typ des zweiten Parameters.</typeparam>
        /// <typeparam name="TParam3">Typ des dritten Parameters.</typeparam>
        /// <typeparam name="TParam4">Typ des vierten Parameters.</typeparam>
        /// <typeparam name="TResult">Typ des Resultats vom Server.</typeparam>
        /// <param name="objInvoke">Methode, die auszuführen ist.</param>
        /// <param name="objParam1">Erster Parameter.</param>
        /// <param name="objParam2">Zweiter Parameter.</param>
        /// <param name="objParam3">Dritter Parameter.</param>
        /// <param name="objParam4">Vierter Parameter.</param>
        /// <returns>Rückgabeobjekt vom Server.</returns>
        protected TResult ExecuteFunc<TParam1, TParam2, TParam3, TParam4, TResult>(
            Func<TParam1, TParam2, TParam3, TParam4, TResult> objInvoke, TParam1 objParam1, TParam2 objParam2, TParam3 objParam3, TParam4 objParam4)
        {
            try
            {
                return objInvoke(objParam1, objParam2, objParam3, objParam4);
            }
            catch (CommunicationException)
            {
                throw new ApplicationException(Properties.Resources.ServerNotAvailable);
            }
        }

        /// <summary>
        /// Startet eine clientseitige asynchrone Methode mit einem Rückgabewert und einem Parameter.
        /// </summary>
        /// <typeparam name="TParam">Typ des Parameters</typeparam>
        /// <typeparam name="TResult">Typ der Rückgabe.</typeparam>
        /// <param name="objWorker">Delgat für die effektive Methodenausführung.</param>
        /// <param name="objCallback">Delegat für den Rückruf, sobald das Resultat des Servs vorliegt.</param>
        /// <param name="objParam">Parameter.</param>
        protected void BeginExecutionAction<TParam, TResult>(Func<TParam, TResult> objWorker, Action<TResult> objCallback, TParam objParam)
        {
            ThreadPool.QueueUserWorkItem(state => AsynchronousExecutor(objWorker, objCallback, objParam));
        }

        /// <summary>
        /// Startet eine clientseitige asynchrone Methode mit einem Rückgabewert und vier Parametern.
        /// </summary>
        /// <typeparam name="TParam1">Typ des ersten Parameters.</typeparam>
        /// <typeparam name="TParam2">Typ des zweiten Parameters.</typeparam>
        /// <typeparam name="TParam3">Typ des dritten Parameters.</typeparam>
        /// <typeparam name="TParam4">Typ des vierten Parameters.</typeparam>
        /// <typeparam name="TResult">Typ des Resultats vom Server.</typeparam>
        /// <param name="objWorker">Delegat der Methode, die effektiv aufgerufen wird.</param>
        /// <param name="objCallback">Delegat für dei Rückmeldung des Resultats.</param>
        /// <param name="objParam1">Erster Parameter.</param>
        /// <param name="objParam2">Zweiter Parameter.</param>
        /// <param name="objParam3">Dritter Parameter.</param>
        /// <param name="objParam4">Vierter Parameter.</param>
        protected void BeginExecutionAction<TParam1, TParam2, TParam3, TParam4, TResult>(
            Func<TParam1, TParam2, TParam3, TParam4, TResult> objWorker, Action<TResult> objCallback, TParam1 objParam1, TParam2 objParam2, TParam3 objParam3, TParam4 objParam4)
        {
            ThreadPool.QueueUserWorkItem(state => AsynchronousExecutor(objWorker, objCallback, objParam1, objParam2, objParam3, objParam4));
        }

        /// <summary>
        /// Methode für die Rückmeldung einer asynchron ausgeführten Methode.
        /// </summary>
        /// <typeparam name="TParam"></typeparam>
        /// <typeparam name="TResult"></typeparam>
        /// <param name="objWorker"></param>
        /// <param name="objCallback"></param>
        /// <param name="objParam"></param>
        private void AsynchronousExecutor<TParam, TResult>(Func<TParam, TResult> objWorker, Action<TResult> objCallback, TParam objParam)
        {
            try
            {
                TResult result = objWorker(objParam);
                if (ClientUIDispatcher.CheckAccess())
                {
                    objCallback(result);
                }
                else
                {
                    ClientUIDispatcher.BeginInvoke(objCallback, result);
                }
            }
            catch (Exception exception)
            {
                HandleException(exception);
            }
        }

        /// <summary>
        /// Methode für die Rückmeldung einer asynchron ausgeführten Methode.
        /// </summary>
        /// <typeparam name="TParam1">Typ des ersten Parameters.</typeparam>
        /// <typeparam name="TParam2">Typ des zweiten Parameters.</typeparam>
        /// <typeparam name="TParam3">Typ des dritten Parameters.</typeparam>
        /// <typeparam name="TParam4">Typ des vierten Parameters.</typeparam>
        /// <typeparam name="TResult">Typ des Resultats vom Server.</typeparam>
        /// <param name="objWorker">Delegat der Methode, die effektiv aufgerufen wird.</param>
        /// <param name="objCallback">Delegat für dei Rückmeldung des Resultats.</param>
        /// <param name="objParam1">Erster Parameter.</param>
        /// <param name="objParam2">Zweiter Parameter.</param>
        /// <param name="objParam3">Dritter Parameter.</param>
        /// <param name="objParam4">Vierter Parameter.</param>
        private void AsynchronousExecutor<TParam1, TParam2, TParam3, TParam4, TResult>(
            Func<TParam1, TParam2, TParam3, TParam4, TResult> objWorker, Action<TResult> objCallback, TParam1 objParam1, TParam2 objParam2, TParam3 objParam3, TParam4 objParam4)
        {
            try
            {
                TResult result = objWorker(objParam1, objParam2, objParam3, objParam4);
                if (ClientUIDispatcher.CheckAccess())
                {
                    objCallback(result);
                }
                else
                {
                    ClientUIDispatcher.BeginInvoke(objCallback, result);
                }
            }
            catch (Exception exception)
            {
                HandleException(exception);
            }
        }

        /// <summary>
        /// Threadwechsel für die Meldugn einer Threadausnahme in den Hauptthread.
        /// </summary>
        /// <param name="objException">Ausnahme, die in den hauptthread übernommen werden soll.</param>
        private void HandleException(Exception objException)
        {
            if (ClientUIDispatcher.CheckAccess())
            {
                if (objException is CommunicationException)
                {
                    throw new ApplicationException(Properties.Resources.ServerNotAvailable);
                }
                else
                {
                    throw objException;
                }
            }
            else
            {
                Action<Exception> exceptionAction = HandleException;
                ClientUIDispatcher.BeginInvoke(exceptionAction, objException);
            }
        }
    }
}
