﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfClientUI
{
    /// <summary>
    /// Interaktionslogik für das Dialogfeld DispatcherDialog.xaml
    /// </summary>
    public partial class DispatcherDialog : Window
    {
        /// <summary>
        /// Zufallszahlengenerator.
        /// </summary>
        private Random _objRandom = new Random();

        /// <summary>
        /// Definiert die Id des Absenders. 
        /// </summary>
        private readonly Guid _objIdSender = Guid.NewGuid();

        /// <summary>
        /// Definiert die Anzahl der abgesendeten Jobs.
        /// </summary>
        private int _iCountJobs;

        /// <summary>
        /// Client der aktuell verwendet wird.
        /// </summary>
        private IwlbDemoDispatcherService _objClient = null;

        /// <summary>
        /// Liefert oder definiert die Jobs auf der Serverseite.
        /// </summary>
        public ObservableCollection<CwlbDispatcherJob> JobCollection { get; private set; }

        /// <summary>
        /// Liefert oder definiert die Clientfactory für die Herstellung des Clients.
        /// </summary>
        public ChannelFactory<IwlbDemoDispatcherService> ClientFactory { get; set; }

        /// <summary>
        /// Initialisiert ein neues Dialogfeld.
        /// </summary>
        public DispatcherDialog()
        {
            JobCollection = new ObservableCollection<CwlbDispatcherJob>();

            CwlbDemoDispatcherCallback objCallback = new CwlbDemoDispatcherCallback();
            objCallback.Done += Execution_Done;
            objCallback.ClientUIDispatcher = this.Dispatcher;
            ClientFactory = new DuplexChannelFactory<IwlbDemoDispatcherService>(objCallback, new NetTcpBinding(), new EndpointAddress("net.tcp://localhost:4711/DispatcherService"));

            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung des Rückrufs des Servers.
        /// </summary>
        /// <param name="objId">Identifikation der erledigten Verarbeitung.</param>
        private void Execution_Done(Guid objId)
        {
            lock (((ICollection)JobCollection).SyncRoot)
            {
                CwlbDispatcherJob objJob = JobCollection.FirstOrDefault(objDone => objDone.Id == objId);
                if (objJob != null)
                {
                    objJob.SetCallbackTime();
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Neu.
        /// </summary>
        private void New_Clicked(object sender, RoutedEventArgs e)
        {
            CheckClient();
            _objClient.Execute(CreateJob());
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Serie.
        /// </summary>
        private void Series_Clicked(object sender, RoutedEventArgs e)
        {
            // Lese Anzahl zu erstellende Aufträge
            int iAmountJobs = Convert.ToInt32(((ComboBoxItem)_cboAmountJobs.SelectedValue).Content);
            CheckClient();
            for (int iCount = 0; iCount < iAmountJobs; iCount++)
            {
                _objClient.Execute(CreateJob());
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Löschen.
        /// </summary>
        private void Clear_Clicked(object sender, RoutedEventArgs e)
        {
            lock (((ICollection)JobCollection).SyncRoot)
            {
                List<CwlbDispatcherJob> cobjJobsFinished = JobCollection.Where(objJobDone => objJobDone.Status == EwlbDispatcherJobState.ReceivedCallback).ToList();
                foreach (CwlbDispatcherJob objJob in cobjJobsFinished)
                {
                    JobCollection.Remove(objJob);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Beenden.
        /// </summary>
        private void Exit_Clicked(object sender, RoutedEventArgs e)
        {
            CloseClient();
            Close();
        }

        /// <summary>
        /// Creates a new dispatcher job.
        /// </summary>
        /// <returns>New created dispatcher job.</returns>
        private CwlbDispatcherJob CreateJob()
        {
            CwlbDispatcherJob objJob = new CwlbDispatcherJob();
            objJob.Id = Guid.NewGuid();
            objJob.Sender = _objIdSender;
            objJob.Name = string.Format("Job #: {0:000-000}", _iCountJobs++);
            objJob.Workload = _objRandom.Next(0, 250);
            objJob.SentFromClient = DateTime.Now;
            objJob.Status = EwlbDispatcherJobState.Sent;
            lock (((ICollection)JobCollection).SyncRoot)
            {
                JobCollection.Add(objJob);
            }
            return objJob;
        }

        /// <summary>
        /// Erstellt einen Client.
        /// </summary>
        private void CheckClient()
        {
            IClientChannel objClientChannel = _objClient as IClientChannel;
            if (objClientChannel == null)
            {
                _objClient = ClientFactory.CreateChannel();
                _objClient.Register(this._objIdSender);
                return;
            }
            else if (objClientChannel.State == CommunicationState.Opened)
            {
                return;
            }
            else if (objClientChannel.State == CommunicationState.Faulted)
            {
                objClientChannel.Abort();
                _objClient = ClientFactory.CreateChannel();
                _objClient.Register(this._objIdSender);
                return;
            }
            else
            {
                _objClient = ClientFactory.CreateChannel();
                _objClient.Register(this._objIdSender);
                return;
            }
        }

        /// <summary>
        /// Closes a client used by this program.
        /// </summary>
        private void CloseClient()
        {
            IClientChannel objClientChannel = _objClient as IClientChannel;
            if (objClientChannel == null)
            {
                return;
            }
            else if (objClientChannel.State == CommunicationState.Opened)
            {
                _objClient.Unregister(_objIdSender);
                objClientChannel.Close();
            }
            else if (objClientChannel.State == CommunicationState.Faulted)
            {
                objClientChannel.Abort();
            }
        }
    }
}
