﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Definiert die Basisfunktionalität für einen Dienst.
    /// </summary>
    public abstract class CwlbDemoService : IwlbDemoService
    {
        /// <summary>
        /// Erstellt ein Synchronisationsobjekt für die Handhabung des Mutlithreadings.
        /// </summary>
        private static object _SyncRoot = new object();

        /// <summary>
        /// Definiert die Basisfunktion der Schnittstelle IDisposable.
        /// </summary>
        public abstract void Dispose();

        /// <summary>
        /// Führt eine einfache Anfrage an den Dienst durch und leifert Maschinedaten.
        /// </summary>
        /// <param name="objPing"></param>
        /// <returns></returns>
        public CwlbPing Ping(CwlbPing objPing)
        {
            // Serverseitige Ausgabe
            WriteLine(string.Format("Ping: {0} - {1}", objPing.ClientName, objPing.ClientSent));

            // Aufbereitung der Daten und Rückgabe des Pings
            objPing.ServerName = Environment.MachineName;
            objPing.ServerSent = DateTime.UtcNow;

            if (objPing.Load > 0)
            {
                objPing.Data = new byte[objPing.Load];
                new Random().NextBytes(objPing.Data);
            }
            return objPing;
        }

        /// <summary>
        /// Schreibt einen übermittelten Text vom Client auf die Konsole.
        /// </summary>
        /// <param name="strGreetingText">Text der vom Client übermittelt wurde.</param>
        public void Greeting(string strGreetingText)
        {
            WriteLine(string.Format("Greeting: {0}", strGreetingText));
        }

        /// <summary>
        /// Methode mit langer Laufzeit.
        /// </summary>
        /// <param name="iDuration">Gewünschte Laufzeit der Methode in Sekunden.</param>
        /// <param name="iAmount">Anzahl der erzeugten Integerwerte.</param>
        /// <param name="iMinValue">Kleinster numerischer Wert der Rückgabe.</param>
        /// <param name="iMaxValue">Grösster numerischer Wert der Rückgabe.</param>
        /// <returns>Array von ganzzahligen Werten der Rückgabe.</returns>
        public int[] LongTermAction(int iDuration, int iAmount, int iMinValue, int iMaxValue)
        {
            WriteLine("Eintritt Long Term Action");
            Thread.Sleep(iDuration * 1000);

            Random objRandom = new Random();
            List<int> cobjData = new List<int>();
            for (int iIndex = 0; iIndex < iAmount; iIndex++)
            {
                cobjData.Add(objRandom.Next(iMinValue, iMaxValue));
            }

            WriteLine("Austritt Long Term Action");
            return cobjData.ToArray();
        }

        /// <summary>
        /// Deklaration der Registrierungsmethode für Rückrufe.
        /// </summary>
        public abstract void Register();

        /// <summary>
        /// Deklaration der Methode für die Aufhebung der Registrierung von Rückrufen.
        /// </summary>
        public abstract void Unregister();

        /// <summary>
        /// Schreibt die gewünschte Ausgabe mit einem Zeitstempel versehen auf die Konsole.
        /// </summary>
        /// <param name="strOut">Ausgabetext.</param>
        protected void WriteLine(string strOut)
        {
            // Die Ausgabe kann multithreaded erfolgen, weild ie Klasse Console threadsicher ist
            Console.WriteLine("{0}: {1}", DateTime.UtcNow.ToString("HH:mm:ss.fff"), strOut);
        }

        #region Serverseitige asynchrone Verarbeitung
        /// <summary>
        /// Serverseitig asynchrone Demomethode.
        /// </summary>
        /// <param name="objData">Datenobjekt das vom Client gesendet wird.</param>
        /// <param name="objAsyncCallback">Rückrufmethode für die Verarbeitung.</param>
        /// <param name="objAsyncState">Statusdaten der Verarbeitung.</param>
        /// <returns>Array von ganzzahligen Werten der Rückgabe.</returns>
        public IAsyncResult BeginLongTermActionFull(CwlbAsyncData objData, AsyncCallback objAsyncCallback, object objAsyncState)
        {
            WriteLine("Eintritt BeginLongTermActionFull");

            // Vorbereiten der Synchronisation der Serverseitigen Aufrufe.
            CwlbAsyncResult<CwlbAsyncData> objAsyncResult = new CwlbAsyncResult<CwlbAsyncData>(false, objAsyncState);

            // Definiere die Ausgabedaten der Verarbeitung
            objAsyncResult.Data = objData;

            // Starte die serverseitige asynchrone Verarbeitung.
            ThreadPool.QueueUserWorkItem(LongTermActionFullWorker, objAsyncResult);

            // Auslösen des Rückrufs für die Beendigung der Arbeit.
            objAsyncCallback(objAsyncResult);

            // Rückgabe des Resultats des Starts der asynchronen Verarbeitung
            WriteLine("Austritt BeginLongTermActionFull");
            return objAsyncResult;
        }

        /// <summary>
        /// Methode für die Beendigung der serverseitigen asynchronen Verarbeitung.
        /// </summary>
        /// <param name="objResult">Resultat der Verarbeitung.</param>
        /// <returns>Ausgabedaten der verarbeitung.</returns>
        public CwlbAsyncData EndLongTermActionFull(IAsyncResult objResult)
        {
            WriteLine("Eintritt EndLongTermActionFull");
            // Umwandlung der allgemeinen Eingabedaten
            CwlbAsyncResult<CwlbAsyncData> objAsyncResult = objResult as CwlbAsyncResult<CwlbAsyncData>;
            if (!objAsyncResult.CompletedSynchronously)
            {
                // Warten bis die Verarbeitung beendet ist.
                if (!objAsyncResult.AsyncWaitHandle.WaitOne(TimeSpan.FromMinutes(1)))
                {
                    // Timeout in der Verarbeitung.
                    objAsyncResult.CancellationPending = true;
                    throw new FaultException<string>("Timeout in der Verarbeitung");
                }
            }

            // Ausnahme im Worker prüfen
            if (objAsyncResult.WorkerException != null)
            {
                // Ausnahme an den Client senden
                throw objAsyncResult.WorkerException;
            }

            // Rückgabe der berechneten Resultate
            WriteLine("Austritt EndLongTermActionFull");
            return objAsyncResult.Data;
        }

        /// <summary>
        /// Arbeitsmethode für die serverseitige asynchrone Verabreitung.
        /// </summary>
        /// <param name="objState"></param>
        private void LongTermActionFullWorker(object objState)
        {
            WriteLine("Eintritt LongTermActionFullWorker");
            CwlbAsyncResult<CwlbAsyncData> objAsyncResult = objState as CwlbAsyncResult<CwlbAsyncData>;
            Random objRandom = new Random();

            try
            {
                // Check state object
                if (objAsyncResult.Data == null)
                {
                    throw new FaultException<string>("Ungültige Verarbeitung", "Der Status enthält keine Daten.");
                }

                // Simuliere die Verarbeitung
                // Berechne das Verarbeitungsende
                DateTime dtEnd = DateTime.UtcNow + TimeSpan.FromSeconds(objAsyncResult.Data.Duration);
                while (dtEnd > DateTime.UtcNow)
                {
                    System.Threading.Thread.Sleep(1000);
                }

                // Berechnen der Rückgabedaten
                objAsyncResult.Data.Data = new int[10];
                for (int iIndex = 0; iIndex < objAsyncResult.Data.Data.Length; iIndex++)
                {
                    objAsyncResult.Data.Data[iIndex] = objRandom.Next(0, 100);
                }
            }
            catch (Exception excObject)
            {
                objAsyncResult.WorkerException = new FaultException<string>(excObject.GetType().Name, excObject.Message);
            }
            finally
            {
                WriteLine("Austritt LongTermActionFullWorker");
                objAsyncResult.Complete();
            }
        }
        #endregion
    }
}
