﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Implementiert den Jobcontroller.
    /// </summary>
    public class CwlbJobController : IDisposable
    {
        /// <summary>
        /// Definiert den Zeitgeber, der für das Starten von neuen Jobs verwendet wird.
        /// </summary>
        private System.Timers.Timer _objJobStarter;

        /// <summary>
        /// Hält die Liste der aktuellen Jobs.
        /// </summary>
        public List<CwlbJobWorker> JobCollection { get; private set; }

        /// <summary>
        /// Liefert oder definiert ein Synchronisierungsobjekt.
        /// </summary>
        public static readonly object Synchronize = new object();

        /// <summary>
        /// Liefert die maximale Anzahl der Jobs die verarbeitet werden sollen.
        /// </summary>
        public int MaxRunningJobs { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz des Job Controllers.
        /// </summary>
        public CwlbJobController()
        {
            JobCollection = new List<CwlbJobWorker>();
            MaxRunningJobs = Environment.ProcessorCount * 2;
            _objJobStarter = new System.Timers.Timer(10000);
            _objJobStarter.Elapsed += JobStarter_Elapsed;
            _objJobStarter.Start();
        }

        /// <summary>
        /// Prüft das Starten von Jobs.
        /// </summary>
        private void JobStarter_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            lock (Synchronize)
            {
                // Anzahl laufende Jobs ermitteln
                int iCount = JobCollection.Count(objJobWorker => objJobWorker.Job.Status == EwlbJobState.Running);
                if (iCount <= MaxRunningJobs)
                {
                    // Starten von neuen Jobs
                    foreach (CwlbJobWorker objWorker in JobCollection
                        .Where(objWorker => objWorker.Job.Status == EwlbJobState.Scheduled)
                        .OrderBy(objWorker => objWorker.Job.SendToService)
                        .Take(MaxRunningJobs - iCount))
                    {
                        objWorker.Start();
                    }
                }
            }
        }

        /// <summary>
        /// Liefert die Liste der aktuellen Jobs im System.
        /// </summary>
        /// <returns>Liste der Jobs.</returns>
        public List<CwlbJob> GetAllJobs()
        {
            lock (Synchronize)
            {
                return JobCollection.Select(objJobWorker => objJobWorker.Job).ToList();
            }
        }

        /// <summary>
        /// Registriert einen neuen Job im System.
        /// </summary>
        /// <param name="objJob">Daten des neuen Jobs.</param>
        public void RegisterJob(CwlbJob objJob)
        {
            lock (Synchronize)
            {
                JobCollection.Add(new CwlbJobWorker(objJob));
            }
        }


        /// <summary>
        /// Löscht einen bestehenden Job aus dem System.
        /// </summary>
        /// <param name="objJobId">Identifikation des zu löschenden Jobs.</param>
        /// <returns>True, wenn der Job gelöscht wurde, anderfalls false.</returns>
        public bool DeleteJob(Guid objJobId)
        {
            lock (CwlbJobController.Synchronize)
            {
                CwlbJobWorker objWorker = JobCollection.FirstOrDefault(objJobWorker => objJobWorker.Job.Id == objJobId);
                if (objWorker == null)
                {
                    return false;
                }
                else
                {
                    switch (objWorker.Job.Status)
                    {
                        case EwlbJobState.Exception:
                        case EwlbJobState.Ended:
                        case EwlbJobState.Scheduled:
                        case EwlbJobState.Cancelled:
                            JobCollection.Remove(objWorker);
                            return true;
                        case EwlbJobState.Running:
                            return false;
                        default:
                            return false;
                    }
                }
            }
        }

        /// <summary>
        /// Hält die Verarbeitung des Dienstes an.
        /// </summary>
        public void Dispose()
        {
            lock (Synchronize)
            {
                // Anhalten des Job starters
                _objJobStarter.Stop();
                _objJobStarter.Dispose();

                // Allen laufenden Jobs mitteilen, dass Sie enden sollen
                foreach (CwlbJobWorker objWorker in JobCollection.Where( objWorker => objWorker.Job.Status == EwlbJobState.Running))
                {
                    objWorker.Job.Status = EwlbJobState.Cancelling;
                }
            }

            // Warten bis alle Jobs beendet sind
            bool bAllEnded = false;
            while (!bAllEnded)
            {
                Thread.Sleep(500);
                lock (Synchronize)
                {
                    int iCountRunning = JobCollection.Count(objWorker => objWorker.Job.Status == EwlbJobState.Running);
                    if (iCountRunning == 0)
                    {
                        bAllEnded = true;
                    }
                }
            }
        }
    }
}
