﻿using System;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Definiert einen Dispatcher für die zentrale Verwaltung der Aufgaben.
    /// </summary>
    sealed class CwlbJobDispatcher
    {
        /// <summary>
        /// Warteschlange für die Verarbeitung der Requests.
        /// </summary>
        private Queue<CwlbDispatcherJobWorker> _cobjJobQueue = new Queue<CwlbDispatcherJobWorker>();

        /// <summary>
        /// Synchronisationselement für das Abbrechen der Verarbeitung des Dispatchers von außen.
        /// </summary>
        public ManualResetEvent Run { get; private set; }

        /// <summary>
        /// Synchronisationselement für die Dispatcherverarbeitung.
        /// </summary>
        private ManualResetEvent _objWorking = new ManualResetEvent(false);

        /// <summary>
        /// Dispatcher Thread.
        /// </summary>
        private Thread _objWorker;

        /// <summary>
        /// Initialisiert eine neue Instanz des Dispatchers.
        /// </summary>
        /// <param name="objService"></param>
        public CwlbJobDispatcher()
        {
            Run = new ManualResetEvent(true);
            _objWorker = new Thread(Execute);
            _objWorker.Name = "Dispatcher Single Thread";
            _objWorker.IsBackground = true;
            _objWorker.Start();
        }

        /// <summary>
        /// Dispatcherthread für die Ausführung von Jobs.
        /// </summary>
        /// <param name="obj">Statusobjekt (nicht benutzt)</param>
        private void Execute(object objStatus)
        {
            CwlbDispatcherJobWorker objCurrentJob = null;

            // Verarbeitung laufen lassen bis sie von aussen gestoppt wird.
            while (Run.WaitOne(0))
            {
                // Warten bis die Verarbeitung signalisiert wird
                if (_objWorking.WaitOne(1000))
                {
                    // Verarbeitung starten
                    lock (((ICollection)_cobjJobQueue).SyncRoot)
                    {
                        // Element aus der Warteschlange nehmen
                        objCurrentJob = _cobjJobQueue.Dequeue();
                        // Prüfen ob noch weitere jobs in der Warteschlange sind
                        if (_cobjJobQueue.Count == 0)
                        {
                            // Abschalten der Verarbeitung
                            _objWorking.Reset();
                        }
                    }

                    // Durchführen der Verarbeitung (Simulation)
                    objCurrentJob.Execute();

                    // Rückruf an den Client auslösen
                    objCurrentJob.OnFinished();
                }
                else
                {
                    // Zeitüberschreitung, prüfen ob abbrechen erwünscht oder 
                    // Logeintrag zur Fehleranalyse vornehmen               
                }
            }
        }

        /// <summary>
        /// Fügt ein Element in die Warteschlange der Jobs ein.
        /// </summary>
        /// <param name="objDispatcherJob">Einzufügendes Element.</param>
        internal void Add(CwlbDispatcherJobWorker objDispatcherJob)
        {
            lock (((ICollection)_cobjJobQueue).SyncRoot)
            {
                // In Warteschlange einfügen
                _cobjJobQueue.Enqueue(objDispatcherJob);
                // Arbeitsthread starten
                _objWorking.Set();
            }
        }
    }
}
