﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;
using System.Threading.Tasks;
using System.Transactions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfClient
{
    /// <summary>
    /// Interaktionslogik für das Hauptfenster des Clients
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Definiert den Namen des Downloadverzeichnisses.
        /// </summary>
        private const string DOWNLOAD = "Download";

        /// <summary>
        /// Liefert eine Liste der Dokumente.
        /// </summary>
        public ObservableCollection<CwlbDocumentHeader> DocumentList { get; private set; }

        /// <summary>
        /// Liefert den Client für den Zugriff auf die serverseitige SChnittstelle für Dokumentenverwaltung.
        /// </summary>
        public IwlbDocumentManager DocumentManager { get; private set; }

        /// <summary>
        /// Definiert ein Hintergundareitsobjekt.
        /// </summary>
        private BackgroundWorker _objBackgroundWorker = new BackgroundWorker();

        /// <summary>
        /// Initialisiert eine neue INstanz des Hauptfensters.
        /// </summary>
        public MainWindow()
        {
            this.DataContext = this;
            DocumentList = new ObservableCollection<CwlbDocumentHeader>();
            InitializeComponent();

            _objBackgroundWorker.WorkerReportsProgress = false;
            _objBackgroundWorker.DoWork += BackgroundWorker_DoWork;
        }

        /// <summary>
        /// Ereignisbehandlung für das Senden der Datei vom Client im Hintergrund.
        /// </summary>
        private void BackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            string strFileName = e.Argument as string;

            IwlbDocumentStream objClientStream = null;
            try
            {
                // Konfigurieren des Clients zum Hochladen
                NetTcpBinding objBinding = new NetTcpBinding();
                objBinding.TransferMode = TransferMode.Streamed;
                objBinding.SendTimeout = TimeSpan.FromMinutes(20.0);
                objClientStream = ChannelFactory<IwlbDocumentStream>.CreateChannel(
                    objBinding,
                    new EndpointAddress(Properties.Settings.Default.AddressStream));
                FileStream streamFile = new FileStream(strFileName, FileMode.Open, FileAccess.Read, FileShare.Read);
                string strTempfileName = objClientStream.UploadDocument(streamFile);

                // ----------------------------------------------------------------------------
                // Hier den Stream nicht schließen, das übernimmt die WCF
                // ----------------------------------------------------------------------------

                // Nach dem Hochladen wird die Datei korrekt registriert
                CwlbDocumentHeader objDocumentHeader = new CwlbDocumentHeader();
                FileInfo objFileInfo = new FileInfo(strFileName);
                objDocumentHeader.Name = objFileInfo.Name;
                objDocumentHeader.Size = objFileInfo.Length;
                objDocumentHeader.Id = strTempfileName;
                DocumentManager.RegisterDocument(objDocumentHeader);
            }
            catch (Exception excObejct)
            {
                ShowException(excObejct);
            }
            finally
            {
                CloseProxy(objClientStream as IChannel);
            }
        }

        /// <summary>
        /// Ereignisbehandlung nach dem Laden des Hauptfensters.
        /// </summary>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                CwlbDocumentManagementCallback objCallback = new CwlbDocumentManagementCallback();
                objCallback.DocumentAdded += DocumentAdded;
                objCallback.DocumentRemoved += DocumentRemoved;
                NetTcpBinding objBinding = new NetTcpBinding();
                DocumentManager = DuplexChannelFactory<IwlbDocumentManager>.CreateChannel(
                    objCallback, 
                    objBinding, new EndpointAddress(Properties.Settings.Default.AddressManagement));
                List<CwlbDocumentHeader> cobjDocuments = DocumentManager.GetDocumentList();
                cobjDocuments.ForEach(objDocument => DocumentList.Add(objDocument));
            }
            catch (Exception excObejct)
            {
                ShowException(excObejct);
            }
        }

        /// <summary>
        /// Zeigt eine Ausnahme an.
        /// </summary>
        /// <param name="excObejct">Ausnahme, deren Text dargestellt werden soll.</param>
        private static void ShowException(Exception excObejct)
        {
            MessageBox.Show(excObejct.Message, "Fehler", MessageBoxButton.OK, MessageBoxImage.Exclamation);
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des Hauptfensters.
        /// </summary>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            CloseProxy(DocumentManager as IChannel);
        }

        /// <summary>
        /// Ereignisbehandlung bei einem Rückruf vom Server für das Hinzufügen eines Dokuments.
        /// </summary>
        private void DocumentAdded(object sender, DocumentEventArgs e)
        {
            if (Dispatcher.CheckAccess())
            {
                DocumentList.Add(e.DocumentHeader);
            }
            else
            {
                EventHandler<DocumentEventArgs> objDelegate = DocumentAdded;
                Dispatcher.BeginInvoke(objDelegate, new object[] { sender, e });
            }
        }

        /// <summary>
        /// Ereignisbehandlung bei einem Rückruf vom Server für das Löschen eines Dokuments.
        /// </summary>
        private void DocumentRemoved(object sender, DocumentEventArgs e)
        {
            if (Dispatcher.CheckAccess())
            {
                CwlbDocumentHeader objHeader = DocumentList.FirstOrDefault(objheader => objheader.Name == e.DocumentHeader.Name);
                if (objHeader != null)
                {
                    DocumentList.Remove(objHeader);
                }
            }
            else
            {
                EventHandler<DocumentEventArgs> objDelegate = DocumentRemoved;
                Dispatcher.BeginInvoke(objDelegate, new object[] { sender, e });
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Beenden.
        /// </summary>
        private void Exit_Clicked(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Hochladen.
        /// </summary>
        private void Upload_Clicked(object sender, RoutedEventArgs e)
        {
            // Dateidialogfeld zur Auswahl einer Datei anzeigen
            OpenFileDialog dlgOpenFile = new OpenFileDialog();
            dlgOpenFile.Filter = Properties.Settings.Default.OpenFileFilter;

            // Überprüfen ob eine Daei ausgewählt wurde und die 
            // Schaltfläche Hochladen betätigt wurde
            if (dlgOpenFile.ShowDialog().GetValueOrDefault())
            {
                _objBackgroundWorker.RunWorkerAsync(dlgOpenFile.FileName);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Auslösen einer Löschoperation.
        /// </summary>
        private void Delete_Clicked(object sender, RoutedEventArgs e)
        {
            CwlbDocumentHeader objDocumentHeader = _lvFiles.SelectedItem as CwlbDocumentHeader;
            if (objDocumentHeader != null)
            {
                DocumentManager.RemoveDocument(objDocumentHeader);
            }
        }

        /// <summary>
        /// Ereignisbehandlung zum Herunterladen und Anzeigen der Datei.
        /// </summary>
        private void Open_Clicked(object sender, RoutedEventArgs e)
        {
            // Datei aus der Liste in der Benutzeroberfläche lesen
            CwlbDocumentHeader objDocumentHeader = _lvFiles.SelectedItem as CwlbDocumentHeader;
            if (objDocumentHeader != null)
            {
                IwlbDocumentStream objClientStream = null;
                try
                {
                    // Konfiguration der Bindung
                    NetTcpBinding objBinding = new NetTcpBinding();
                    objBinding.TransferMode = TransferMode.Streamed;
                    objBinding.MaxReceivedMessageSize = 100000000000;

                    // Herstellen des Clients
                    objClientStream = ChannelFactory<IwlbDocumentStream>.CreateChannel(
                        objBinding,
                        new EndpointAddress(Properties.Settings.Default.AddressStream));

                    // Prüfen ob das Zielverzeichnis exisitiert und anlegen wenn notwendig
                    if (!Directory.Exists(DOWNLOAD))
                    {
                        Directory.CreateDirectory(DOWNLOAD);
                    }

                    // Voll qualifizierten Dateinamen herstellen und Datei vom Server laden
                    string strLocalFileName = System.IO.Path.Combine(DOWNLOAD, objDocumentHeader.Name);
                    using (Stream streamFromServer = objClientStream.DownloadDocument(objDocumentHeader.Name))
                    using (FileStream streamFile = new FileStream(strLocalFileName, FileMode.OpenOrCreate, FileAccess.Write))
                    {
                        int iBufferSize = 8192;
                        byte[] abyteStreamed = new byte[iBufferSize];
                        int iBytesRead = streamFromServer.Read(abyteStreamed, 0, iBufferSize);
                        while (iBytesRead > 0)
                        {
                            streamFile.Write(abyteStreamed, 0, iBytesRead);
                            iBytesRead = streamFromServer.Read(abyteStreamed, 0, iBufferSize);
                        }
                    }

                    // Datei im Medai player anzeigen
                    _mediaElement.Source = new Uri(strLocalFileName, UriKind.Relative);
                    _mediaElement.LoadedBehavior = MediaState.Manual;
                    _mediaElement.Play();

                }
                catch (Exception excObejct)
                {
                    ShowException(excObejct);
                }
                finally
                {
                    CloseProxy(objClientStream as IChannel);
                }
            }
        }

        /// <summary>
        /// Schliesst einen Proxy.
        /// </summary>
        private static void CloseProxy(IChannel objChannel)
        {
            if (objChannel == null)
            {
                return;
            }

            if (objChannel.State == CommunicationState.Opened)
            {
                objChannel.Close();
            }
            else if (objChannel.State == CommunicationState.Faulted)
            {
                objChannel.Abort();
            }
        }
    }
}
