﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.Text;
using System.Threading.Tasks;

namespace WeroSoft.Samples.LearningWcfContract
{
    /// <summary>
    /// Implementiert eine Parameterprüfung von Parametern einer Operation, die keinen Null-Wert annehmen dürfen.
    /// </summary>
    public sealed class CwlbParameterNotNullInspector : IParameterInspector
    {
        /// <summary>
        /// Cache für die Relexionsinformation der Parameter der Methode.
        /// </summary>
        private ParameterInfo[] _aobjParameterInfo;

        /// <summary>
        /// Paramater, die überprüft werden sollen.
        /// </summary>
        public string[] ParameterNotNull { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbParameterNotNullInspector.
        /// </summary>
        /// <param name="objMethodInfo">Beschreibung der Methode, die mit diesem Inspektor überwacht werden soll.</param>
        /// <param name="astrParam">Array der Parameternamen, die nicht null sein dürfen.</param>
        public CwlbParameterNotNullInspector(MethodInfo objMethodInfo, string[] astrParam)
        {
            // Erstellen des Zwischenspeichers für die Reflexionsinformationen der Parameter
            _aobjParameterInfo = objMethodInfo.GetParameters();
            // Übernehmen der zu prüfenden Parameter
            ParameterNotNull = astrParam;
        }

        /// <summary>
        /// Prüfmethode nach dem Aufruf der Methode.
        /// </summary>
        /// <param name="objOperationName">Name der aufgerufenen Methode.</param>
        /// <param name="aobjOutputs">Liste der Ausgabeparaemter.</param>
        /// <param name="objReturnValue">Rückgabewert der Methode.</param>
        /// <param name="objCorrelationState">Korrelationsidentifikation.</param>
        public void AfterCall(string objOperationName, object[] aobjOutputs, object objReturnValue, object objCorrelationState)
        {
            Console.WriteLine(
                "Nach Aufruf: {0} liefert den Wert: {1}",
                objOperationName,
                objReturnValue == null ? "null" : objReturnValue.ToString());
        }

        /// <summary>
        /// Prüfmethode vor dem Aufruf der Methode beim Dienst.
        /// </summary>
        /// <param name="objOperationName">Name der aufgerufenen Methode.</param>
        /// <param name="objInputs">Array mit den Parametern der Methode.</param>
        /// <returns>
        /// Ein Objekt, das zur Korelation der Argumente nach dem Methodenaufruf AfterReceiveReply verwendet wird. Der Return dieser Methode
        /// ist null, wenn keine Korrelation verwendet werden soll.
        /// </returns>
        public object BeforeCall(string objOperationName, object[] objInputs)
        {
            Console.Write("Vor Aufruf: {0}(", objOperationName);
            int iIndex = 0;
            string strFirstWrongParametername = null;
            try
            {
                // Durchlaufen aller Parameter gemäß Reflexion
                foreach (ParameterInfo objInfo in _aobjParameterInfo)
                {
                    // Erstellen einer Ausgabe in der Konsole
                    if (iIndex > 0)
                    {
                        Console.Write(", ");
                    }

                    Console.Write(
                        "{0} = {1}", 
                        objInfo.Name, 
                        objInputs[iIndex] == null ? "null" : objInputs[iIndex].ToString());

                    // Null-Prüfung vornehmen, sofern der Parameter in der Liste ist
                    if (_aobjParameterInfo.FirstOrDefault(
                        objParam => objParam.Name == objInfo.Name) != null)
                    {
                        if (string.IsNullOrEmpty(strFirstWrongParametername) &&
                            objInputs[iIndex] == null)
                        {
                            strFirstWrongParametername = objInfo.Name;
                        }
                    }

                    // Inkrement des Index für den Arrayzugriff
                    iIndex++;
                }

                // Prüfen ob ein Fehlerhafter Parameter entdeckt wurde:
                if (!string.IsNullOrEmpty(strFirstWrongParametername))
                {
                    throw new ArgumentNullException(strFirstWrongParametername);
                }
            }
            finally
            {
                Console.WriteLine(")");
            }
            return null;
        }
    }
}
