﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Hilfsklasse für die Konfiguration der Anwendung
    /// </summary>
    public class CwlbArgumentHelper
    {
        // Konstanten für die Schematypen
        private const string BINDING_TCP = "net.tcp";
        private const string BINDING_HTTP = "http";
        private const string BINDING_HTTPS = "https";

        /// <summary>
        /// Liefert oder definiert den Sicherheitsmodus.
        /// </summary>
        public SecurityMode SecurityMode { get; set; }

        /// <summary>
        /// Liefrt oder definiert die Bindung (Schematyp)
        /// </summary>
        public string Binding { get; set; }

        /// <summary>
        /// Liefert oder definiert den verwendeten Port.
        /// </summary>
        public int Port { get; set; }

        /// <summary>
        /// Liefert oder definiert den Maschinennamen.
        /// </summary>
        public string MachineName { get; set; }

        /// <summary>
        /// Liefert oder definiert die verwendeten Credentials.
        /// </summary>
        public HttpClientCredentialType ClientCredentialType { get; set; }

        /// <summary>
        /// Liefert oder definiert denFingerabdruck des zu verwendenden Zertifikates.
        /// </summary>
        public string Thumbprint { get; set; }

        /// <summary>
        /// Liefert oder definiert true, wenn die Hilfe angezeigt werden soll und das Programm abgebrochen wird.
        /// </summary>
        public bool Help { get; set; }

        /// <summary>
        /// Liest die Parameter und konfiguriert das Parameterobjekt
        /// </summary>
        /// <param name="astrArguments"></param>
        public void Parse(string[] astrArguments)
        {
            // Standardargument konfigurieren
            SecurityMode = System.ServiceModel.SecurityMode.None;
            Binding = BINDING_TCP;
            Port = 4711;
            MachineName = "localhost";
            Help = false;
            Thumbprint = "44f63c707a7587ba770be2de1973aa9319e5f3ea";

            // Argument parsen
            foreach (string strArgument in astrArguments)
            {
                try
                {
                    switch (strArgument.ToLower().Substring(1, 1))
                    {
                        // Sicherheit
                        case "s":
                            SecurityMode = (SecurityMode)Enum.Parse(typeof(SecurityMode),strArgument.Substring(3));
                            break;

                        // Port
                        case "p":
                            Port = int.Parse(strArgument.Substring(3));
                            break;

                        // Bindung
                        case "b":
                            Binding = strArgument.Substring(3).ToLower();
                            break;

                        // Maschinenname
                        case "m":
                            MachineName = strArgument.Substring(3);
                            break;

                        // Credentials festlegen
                        case "c":
                            ClientCredentialType = (HttpClientCredentialType)Enum.Parse(typeof(HttpClientCredentialType), strArgument.Substring(3));
                            break;

                        case "t":
                            Thumbprint = strArgument.Substring(3);
                            break;

                        // Hilfe anfordern
                        case "?":
                        default:
                            Help = true;
                            return;
                    }
                }
                catch (Exception excObject)
                {
                    Console.WriteLine("Argumentfehler: {0}", excObject.Message);
                }
            }

            CheckParams();
        }

        /// <summary>
        /// Überprüft die eingelesenen Parameter auf ihre Gültigkeit.
        /// </summary>
        private void CheckParams()
        {
            if (Binding != BINDING_TCP && Binding != BINDING_HTTP && Binding != BINDING_HTTPS)
            {
                Console.WriteLine("Ungültige Bindung definiert.");
                Help = true;
            }

            if (Port < 1024)
            {
                Console.WriteLine("Ungültiger Port definiert.");
                Help = true;
            }
        }

        /// <summary>
        /// Erzeugt und konfiguriert das Bindungsobjekt.
        /// </summary>
        /// <returns></returns>
        public Binding CreateBinding()
        {
            Binding objBinding = null;

            switch (Binding)
            {
                case BINDING_HTTP:
                    switch (SecurityMode)
                    {
                        case SecurityMode.None:
                            objBinding = new BasicHttpBinding(BasicHttpSecurityMode.None);
                            break;
                        case SecurityMode.Transport:
                            objBinding = new BasicHttpBinding(BasicHttpSecurityMode.Transport);
                            break;
                        case SecurityMode.Message:
                            objBinding = new BasicHttpBinding(BasicHttpSecurityMode.Message);
                            break;
                        case SecurityMode.TransportWithMessageCredential:
                            objBinding = new BasicHttpBinding(BasicHttpSecurityMode.TransportWithMessageCredential);
                            break;
                        default:
                            throw new NotSupportedException("Die Bindung wird nicht unterstützt.");
                    }
                    SetCredentialsHttp(objBinding as BasicHttpBinding);
                    break;

                case BINDING_HTTPS:
                    objBinding = new WSHttpBinding(SecurityMode);
                    SetCredentialsWsHttp(objBinding as WSHttpBinding);
                    break;

                default:
                case BINDING_TCP:
                    objBinding = new NetTcpBinding(SecurityMode);
                    SetCredentialsTcp(objBinding as NetTcpBinding);
                    break;
            }

            // Return the Binding
            return objBinding;
        }

        /// <summary>
        /// Definiert die Anmeldeinformationen des Clients.
        /// </summary>
        /// <param name="objBinding">Bindungsobjekt das zu konfigurieren ist.</param>
        private void SetCredentialsWsHttp(WSHttpBinding objBinding)
        {
            objBinding.Security.Transport.ClientCredentialType = ClientCredentialType;
        }

        /// <summary>
        /// Definiert die Anmeldeinformationen des Clients.
        /// </summary>
        /// <param name="objBinding">Bindungsobjekt das zu konfigurieren ist.</param>
        private void SetCredentialsHttp(BasicHttpBinding objBinding)
        {
            objBinding.Security.Transport.ClientCredentialType = ClientCredentialType;
        }

        /// <summary>
        /// Definiert die Anmeldeinformationen des Clients.
        /// </summary>
        /// <param name="objBinding">Bindungsobjekt das zu konfigurieren ist.</param>
        private void SetCredentialsTcp(NetTcpBinding objBinding)
        {
            switch (ClientCredentialType)
            {
                case HttpClientCredentialType.Certificate:
                    objBinding.Security.Transport.ClientCredentialType = TcpClientCredentialType.Certificate;
                    objBinding.Security.Transport.ProtectionLevel = System.Net.Security.ProtectionLevel.EncryptAndSign;
                    break;
                case HttpClientCredentialType.None:
                    objBinding.Security.Transport.ClientCredentialType = TcpClientCredentialType.None;
                    break;
                case HttpClientCredentialType.Windows:
                    objBinding.Security.Transport.ClientCredentialType = TcpClientCredentialType.Windows;
                    objBinding.Security.Transport.ProtectionLevel = System.Net.Security.ProtectionLevel.EncryptAndSign;
                    break;
                case HttpClientCredentialType.Basic:
                case HttpClientCredentialType.Digest:
                case HttpClientCredentialType.Ntlm:
                default:
                    throw new NotSupportedException("Die Authentifizierung wird nicht unterstützt.");
            }
        }

        /// <summary>
        /// Erzeugt die zu verwendende Adresse.
        /// </summary>
        /// <returns>Adresse mit Schema und Port.</returns>
        public string CreateAddress()
        {
            return string.Format("{0}://{1}:{2}", Binding, MachineName, Port);
        }

        /// <summary>
        /// Zeigt die Onlinehilfe für dieses Programm an.
        /// </summary>
        public void ShowHelp()
        {
            Console.WriteLine();
            Console.WriteLine("   Learning WCF - Hilfe");
            Console.WriteLine("   --------------------");
            Console.WriteLine();
            Console.WriteLine("   Folgende Argumente sind unterstützt");
            Console.WriteLine("      -s:(None) | Message | Transport");
            Console.WriteLine("      -p:Portnummer");
            Console.WriteLine("      -b:net.tcp|http");
            Console.WriteLine("      -m:Maschinenname | IP-Adresse | localhost");
            Console.WriteLine("      -c:Windows | None | Certificate");
            Console.WriteLine("      -t:Fingerabdruck des verwendeten Zertifikates");
            Console.WriteLine("      -?");
            Console.WriteLine();
            Console.WriteLine("   Beispiel: -b:net.tcp -m:172.16.172.2 -p:4711 -s:Transport");
            Console.WriteLine();
        }

        /// <summary>
        /// Zeigt die aktuelle Konfiguration
        /// </summary>
        public void ShowConfiguration()
        {
            Console.WriteLine();
            Console.WriteLine("   Learning WCF - Verwendete Konfiguration");
            Console.WriteLine("   ---------------------------------------");
            Console.WriteLine("   Bindung: . . . . . {0}", Binding);
            Console.WriteLine("   Maschinenname: . . {0}", MachineName);
            Console.WriteLine("   Port:. . . . . . . {0}", Port);
            Console.WriteLine("   Sicherheitsmodus:  {0}", SecurityMode);
            Console.WriteLine("   Authentifizierung: {0}", ClientCredentialType);
            Console.WriteLine();
        }
    }
}
