﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WeroSoft.Samples.AssertLibrary
{
    /// <summary>
    /// Prüft einen WCF Datenvertrag auf projektspezifische Korrektheit.
    /// </summary>
    public static class CwlbDataContractAssert
    {
        /// <summary>
        /// Prüft einen Typ auf die korrekte Verwendung eines Datenvertrags
        /// </summary>
        /// <param name="typeValidate">Typ der geprüft werden soll.</param>
        /// <param name="cstrExcludedMember">Liste der Elemente, die nicht geprüft werden sollen.</param>
        public static void IsValidDataContract(Type typeValidate, List<string> cstrExcludedMember)
        {
            // Testen ob die Klasse ein gültiger Datenvertragstyp ist
            if (!(typeValidate.IsClass || typeValidate.IsValueType || typeValidate.IsEnum))
            {
                throw new AssertFailedException("Der übergebene Typ ist kein gültiger Datenvertragstyp.");
            }

            // Feststellen des Prüfung, die vorgenommen werden soll
            object[] aobjAttribute = typeValidate.GetCustomAttributes(typeof(DataContractAttribute), false);
            if (aobjAttribute.Length == 0)
            {
                // Fall ohne Vertragsattribut
                IsValidImplicit(typeValidate, cstrExcludedMember);
            }
            else
            {
                // Fall mit Vertragsattribut
                IsValidExplicit(typeValidate, aobjAttribute[0] as DataContractAttribute, cstrExcludedMember);
            }
        }

        #region Helpermethods for method IsValidDataContract
        /// <summary>
        /// Prüft die korrekte Definition, wenn kein Attribut für den Datenvertrag definiert ist.
        /// </summary>
        /// <param name="typeValidate">Typ der geprüft werden soll.</param>
        /// <param name="cstrExcludedMember">Liste der Elemente, die nicht geprüft werden sollen.</param>
        private static void IsValidImplicit(Type typeValidate, List<string> cstrExcludedMember)
        {
            // Prüft die Exkludeliste
            if (cstrExcludedMember != null && cstrExcludedMember.Count > 0)
            {
                throw new AssertFailedException("Bei einer impliziten Vertagsdefinition muss die Liste der Ausnahmemitglieder leer sein.");
            }

            // Testen, ob die Klasse öffentlich ist und mindestens ein öffentliches Feld oder eine öffentliche
            // Eigenschaft hat
            if (!typeValidate.IsPublic)
            {
                throw new AssertFailedException("Die Klasse ist nicht öffentlich.");
            }

            int iCountPublicMember = 0;
            foreach (MemberInfo objInfo in typeValidate.GetMembers())
            {
                switch (objInfo.MemberType)
                {
                    default:
                        continue;
                    case MemberTypes.Field:
                        iCountPublicMember += CheckImplizitField(objInfo as FieldInfo);
                        break;
                    case MemberTypes.Property:
                        iCountPublicMember += CheckImplizitProperty(objInfo as PropertyInfo);
                        break;
                }
            }

            // Abschließende Prüfung
            if (iCountPublicMember < 1)
            {
                throw new AssertFailedException("Der Datenvertrag enthält nicht mindestens ein gültiges öffentliches Element.");
            }
        }

        /// <summary>
        /// Prüft eine implizite Eigenschaft eines impliziten Datenvertrags.
        /// </summary>
        /// <param name="objPropertyInfo">Informationsobjekt der Eigenschaft.</param>
        /// <returns>Liefert 1, wenn die Eigenschaft gültig ist für einen impliziten Vertrag, ansonsten 0.</returns>
        private static int CheckImplizitProperty(PropertyInfo objPropertyInfo)
        {
            // Prüfe die Verwendung des Attributs DataMemeber
            CheckDataMemberNotDefined(objPropertyInfo);
            if (objPropertyInfo.CanRead && objPropertyInfo.CanWrite)
            {
                // Die Eigesnchaft unterstützt lesen und schreiben
                return 1;
            }
            else
            {
                // Die Eigenschaft unterstützt nicht beide Handhabungsrichtungen
                return 0;
            }
        }

        /// <summary>
        /// Prüft ein implizites Feld eines impliziten Datenvertrags.
        /// </summary>
        /// <param name="objFieldInfo">Informationsobjekt des Feldes.</param>
        /// <returns>Liefert 1, wenn das Feld gültig ist für einen impliziten Vertrag, ansonsten 0.</returns>
        private static int CheckImplizitField(FieldInfo objFieldInfo)
        {
            // Prüfe die Verwendung des Attributs DataMemeber
            CheckDataMemberNotDefined(objFieldInfo);
            if (objFieldInfo.IsPublic)
            {
                return 1;
            }
            else
            {
                return 0;
            }
        }

        /// <summary>
        /// Prüft eine implizite Eigenschaft auf die nicht-Definition des Attributs DataMember.
        /// </summary>
        /// <param name="objInfo">Informationsobjekt, das geprüft werden soll.</param>
        private static void CheckDataMemberNotDefined(MemberInfo objInfo)
        {
            DataMemberAttribute objAttribute = objInfo.GetCustomAttribute<DataMemberAttribute>();
            if (objAttribute != null)
            {
                throw new AssertFailedException("Die Verwendung des Attributes DataMember ist bei impliziter Definition eines Datenvertrags ungültig.");
            }
        }

        /// <summary>
        /// Prüft die korrekte Definition, wenn ein Attribut für den Datenvertrag definiert ist.
        /// </summary>
        /// <param name="typeValidate">Typ der geprüft werden soll.</param>
        /// <param name="objDataContractAttribute">Atttribut, das den Datenvertrag beschreibt.</param>
        /// <param name="cstrExcludedMember">Liste der Elemente, die nicht geprüft werden sollen.</param>
        private static void IsValidExplicit(Type typeValidate, DataContractAttribute objDataContractAttribute, List<string> cstrExcludedMember)
        {
            // Testen, ob die Klasse öffentlich ist und mindestens ein öffentliches Feld oder eine öffentliche
            // Eigenschaft hat
            if (!typeValidate.IsPublic)
            {
                throw new AssertFailedException("Die Klasse ist nicht öffentlich.");
            }

            int iCountPublicMember = 0;
            foreach (MemberInfo objInfo in typeValidate.GetMembers())
            {
                // Prüfen ob das Element in der Excludeliste definiert ist
                if (cstrExcludedMember != null && cstrExcludedMember.Contains(objInfo.Name))
                {
                    continue;
                }

                // Prüfen des Informationstyps
                switch (objInfo.MemberType)
                {
                    default:
                        continue;
                    case MemberTypes.Field:
                        iCountPublicMember += CheckExplicitField(objInfo as FieldInfo);
                        break;
                    case MemberTypes.Property:
                        iCountPublicMember += CheckExplicitProperty(objInfo as PropertyInfo);
                        break;
                }
            }

            // Abschließende  Prüfung
            if (iCountPublicMember < 1)
            {
                throw new AssertFailedException("Der Datenvertrag enthält nicht mindestens ein gültiges öffentliches Element.");
            }
        }

        /// <summary>
        /// Prüft eine Eigenschaft auf gültige Definition des Attributs DataMemberAttribute.
        /// </summary>
        /// <param name="objPropertyInfo">Informationsobjekt der zu prüfenden Eigenschaft.</param>
        /// <returns>Liefert 1, wenn die Eigenschaft gültig ist, 0 wenn die Eigenachft korrekt aber nicht öffentlich definiert ist.</returns>
        private static int CheckExplicitProperty(PropertyInfo objPropertyInfo)
        {
            if (objPropertyInfo.CanRead && objPropertyInfo.CanWrite)
            {
                CheckPublicDataMemberIsDefined(objPropertyInfo);
                return 1;
            }
            else
            {
                CheckNonPublicDataMemberIsDefined(objPropertyInfo);
            }

            return 0;
        }

        /// <summary>
        /// Prüft ein Feld auf gültige Definition des Attributs DataMemberAttribute.
        /// </summary>
        /// <param name="objFieldInfo">Informationsobjekt des zu prüfenden Felds.</param>
        /// <returns>Liefert 1, wenn das Feld gültig ist, 0 wenn das Feld korrekt aber nicht öffentlich definiert ist.</returns>
        private static int CheckExplicitField(FieldInfo objFieldInfo)
        {
            if (objFieldInfo.IsPublic)
            {
                // Prüfen ob das Feld korrekt als Datenmember deklariert ist
                CheckPublicDataMemberIsDefined(objFieldInfo);
                return 1;
            }

            // Des Feld ist nicht öffentlich
            return 0;
        }

        /// <summary>
        /// Prüft ein Element auf korrekte Anwendung des Attributs DataMemberAttribut bei öffentlichen Elementen.
        /// </summary>
        /// <param name="objInfo">Zu prüfendes Informationsobjekt.</param>
        private static void CheckPublicDataMemberIsDefined(MemberInfo objInfo)
        {
            DataMemberAttribute objAttribute = objInfo.GetCustomAttribute<DataMemberAttribute>();
            if (objAttribute == null)
            {
                throw new AssertFailedException("Ein öffentliches Element in einem expliziten Datenvertrag muss mit dem Attribut DataMember gekennzeichnet sein.");
            }

            // Fügen Sie Code ein für die Inhaltsprüfung des Attributs
            // Zum Beispiel Sicherstellung, dass der Name definiert ist, dass der Namensraum korrekt ist
            // oder dass eine Reihenfolge definiert ist.
        }

        /// <summary>
        /// Prüft ein Element auf korrekte Anwendung des Attributs DataMemberAttribut bei nicht öffentlichen Elementen.
        /// </summary>
        /// <param name="objInfo">Zu prüfendes Informationsobjekt.</param>
        private static void CheckNonPublicDataMemberIsDefined(MemberInfo objInfo)
        {
            DataMemberAttribute objAttribute = objInfo.GetCustomAttribute<DataMemberAttribute>();
            if (objAttribute != null)
            {
                throw new AssertFailedException("Ein nicht öffentliches Element in einem expliziten Datenvertrag soll nicht mit dem Attribut DataMember gekennzeichnet sein.");
            }
        }
        #endregion

        /// <summary>
        /// Prüfmethode für die Serialisierung von Daten.
        /// </summary>
        /// <typeparam name="T">Typ der Daten die Serialisiert werden sollen.</typeparam>
        /// <param name="objData">Daten die serialisiert werden sollen.</param>
        /// <param name="iMaxSize">Maximal zulässige Grösse der Datenserialisierung.</param>
        public static T IsDataContractSerializable<T>(T objData, int iMaxSize = int.MinValue)
        {
            T objDeserialized = default(T);
            using (var stream = new System.IO.MemoryStream())
            {
                // Benützung des Data Contract Serializer um zu vermeiden, dass das Attribut
                // SerializableAttribute verwendet werden muss
                var formatter = new System.Runtime.Serialization.NetDataContractSerializer();
                formatter.Serialize(stream, objData);
                stream.Position = 0;

                // Prüfen der maximalen Größe, wenn verlangt
                if (iMaxSize > 0)
                {
                    if (stream.Length > iMaxSize)
                    {
                        throw new AssertFailedException("Die maximal zulässige Datengröße wurde überschritten."); 
                    }
                }
                objDeserialized = (T)formatter.Deserialize(stream);
            }

            // Prüfen des Resultats
            Assert.AreNotSame(objData, objDeserialized, "Das Objekt vor und nach der Serialisierung sind identisch.");

            // Fügen Sie hier beliebige zusätzliche Prüfungen hinzu,
            // oder führen Sie die Kontrollen außerhalb durch.

            // Rückgabe des deserialisierten Objekts
            return objDeserialized;
        }
    }
}
