﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Klasse für die Demonstration des Verhaltens der Client-Server-Technik.
    /// </summary>
    [Serializable]
    public class CwlbDemoData
    {
        /// <summary>
        /// Defininiert die domänenspezifische Stoppuhr.
        /// </summary>
        private static Stopwatch _objStopwatch = Stopwatch.StartNew();

        /// <summary>
        /// Objekt für die Synchronizierung der Instanzennummer.
        /// </summary>
        private static object SnychronizationObject = new object();

        /// <summary>
        /// Liefert oder definiert die Objektidentifikation.
        /// </summary>
        public Guid ID { get; set; }

        /// <summary>
        /// Liefert oder definiert die Anzahl der Instanzen.
        /// </summary>
        public static int InstanceCounter { get; set; }

        /// <summary>
        /// Liefert oder definiert den Namen des Objekts.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Liefert oder definierz einen zufälligen ganzzahligen Wert.
        /// </summary>
        public int IntValue { get; set; }

        /// <summary>
        /// Liefert oder definiert eine zufällige Gleitkommazahl des Typs double.
        /// </summary>
        public double DoubleValue { get; set; }

        /// <summary>
        /// Liefert oder definiert eine zufällige Gleitkommazahl des Typs decimal.
        /// </summary>
        public decimal DecimalValue { get; set; }

        /// <summary>
        /// Liefert oder definiert zufällige, binäre Daten.
        /// </summary>
        public byte[] BinaryData { get; set; }

        /// <summary>
        /// Liefert oder definiert den Namen der Serverdomäne.
        /// </summary>
        public string ServersAppDomainName { get; set; }

        /// <summary>
        /// Zeitpunkt der Erzeugung des Objekts.
        /// </summary>
        public DateTime Creation { get; set; }

        /// <summary>
        /// Zeitpunkt beim Servereintritt.
        /// </summary>
        public TimeSpan ServerEntry { get; set; }

        /// <summary>
        /// Zeitpunkt beim Serveraustritt.
        /// </summary>
        public TimeSpan ServerExit { get; set; }

        /// <summary>
        /// Liefert die Zeit, die für dei Bearbeitung auf dem Server benutzt wurde.
        /// </summary>
        public TimeSpan ServerTime
        {
            get
            {
                return ServerExit - ServerEntry;
            }
        }

        /// <summary>
        /// Liefert oder definiert den Status des Objekts auf dem Roundtrip.
        /// </summary>
        public EwlbRoundtripState RoundtripState { get; private set; }

        /// <summary>
        /// Initialisiert ein neues Objekt der Klasse CwlbCsDemoData.
        /// </summary>
        public CwlbDemoData()
        {
            lock (CwlbDemoData.SnychronizationObject)
            {
                InstanceCounter++;
            }
        }

        /// <summary>
        /// Erzeugt ein Demodatenobjekt.
        /// </summary>
        /// <param name="objRandom">Instanz des Zufallszahlen generators für die Erstellung von zufälligen Daten.</param>
        /// <param name="iBytes">Definiert die Grösse des zu initialisierenden Byte-Arrays.</param>
        /// <returns>Liefert ein initialisiertes Demodatenobjekt.</returns>
        public static CwlbDemoData Create(Random objRandom, int iBytes = 0)
        {
            CwlbDemoData objData = new CwlbDemoData();
            objData.ID = Guid.NewGuid();
            objData.Name = string.Format("Objekt {0}", objData.ID);
            objData.Creation = DateTime.Now;
            objData.IntValue = objRandom.Next();
            objData.DoubleValue = objRandom.NextDouble();
            objData.DecimalValue = (decimal)objRandom.NextDouble();
            if (iBytes > 0)
            {
                objData.BinaryData = new byte[iBytes];
                objRandom.NextBytes(objData.BinaryData);
            }

            return objData;
        }

        /// <summary>
        /// Wird beim deserialisieren des Objekts aufgerufen.
        /// </summary>
        /// <param name="context">Kontext für die Serialisierung.</param>
        [OnDeserialized]
        public void OnDeserialized(StreamingContext context)
        {
            lock (CwlbDemoData.SnychronizationObject)
            {
                InstanceCounter++;
            }
        }

        /// <summary>
        /// Modifiziert ein Demodatenobjekt.
        /// </summary>
        /// <param name="objData">Objekt das Verändert werden soll.</param>
        /// <returns>Liefert das veränderte Datenobjekt.</returns>
        public static CwlbDemoData Modify(CwlbDemoData objData)
        {
            lock (CwlbDemoData.SnychronizationObject)
            {
                try
                {
                    objData.ServerEntry = _objStopwatch.Elapsed;
                    objData.ServersAppDomainName = AppDomain.CurrentDomain.FriendlyName;
                    if (objData.BinaryData != null && objData.BinaryData.Length > 1)
                    {
                        Array.Reverse(objData.BinaryData);
                    }
                    return objData;
                }
                finally
                {
                    objData.ServerExit = _objStopwatch.Elapsed;
                }
            }
        }
    }
}
